=== modified file 'NEWS'
--- NEWS	2005-09-12 09:47:33 +0000
+++ NEWS	2005-09-26 08:56:15 +0000
@@ -2,12 +2,50 @@
 
   IMPROVEMENTS:
 
+    * Status now accepted a -r argument to give status between chosen
+      revisions. Contributed by Heikki Paajanen.
+
+    * Revision arguments no longer use +/-/= to control ranges, instead
+      there is a 'before' namespace, which limits the successive namespace.
+      For example '$ bzr log -r date:yesterday..before:date:today' will
+      select everything from yesterday and before today. Contributed by
+      Robey Pointer
+
+    * There is now a bzr.bat file created by distutils when building on 
+      Windows. Contributed by Alexander Belchenko.
+
+  BUG FIXES:
+
+    * Various minor bug in windows support have been fixed, largely in the
+      test suite. Contributed by Alexander Belchenko.
+      
+  INTERNALS:
+
+    * removed uuid() as it was unused.
+
+bzr 0.0.8 2005-09-20
+
+  IMPROVEMENTS:
+
     * Adding a file whose parent directory is not versioned will
       implicitly add the parent, and so on up to the root. This means
       you should never need to explictly add a directory, they'll just
       get added when you add a file in the directory.  Contributed by
       Michael Ellerman.
 
+    * Ignore .DS_Store (contains Mac metadata) by default.  Patch from
+      Nir Soffer.
+
+    * If you set BZR_EDITOR in the environment, it is checked in
+      preference to EDITOR and the config file for the interactive commit
+      editing program. Related to this is a bugfix where a missing program
+      set in EDITOR would cause editing to fail, now the fallback program
+      for the operating system is still tried.
+
+    * Files that are not directories/symlinks/regular files will no longer
+      cause bzr to fail, it will just ignore them by default. You cannot add
+      them to the tree though - they are not versionable.
+
 
   INTERNALS:
 
@@ -29,6 +67,8 @@
 
     * Various Windows fixes from Ollie Rutherfurd.
 
+    * Quieten warnings about locking; patch from Matt Lavin.
+
 
 bzr-0.0.7 2005-09-02
 

=== modified file 'TODO'
--- TODO	2005-09-12 15:06:23 +0000
+++ TODO	2005-09-19 02:52:24 +0000
@@ -24,6 +24,11 @@
 
   2005-11-05      mbp  Flowers for stephane
 
+* Fix tests so that import errors caused by modules don't produce false reports
+  that the tests themselves don't exist.
+
+* Fix tests so that one test failure doesn't prevent other tests from running
+
 * print a message at the end of running the tests telling them that the
   test log and output exists but can be removed
 

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/__init__.py	2005-09-23 03:22:16 +0000
@@ -28,17 +28,19 @@
                   'BitKeeper',
                   '.git',
                   'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*']
+                  '.del-*',
+                  '.DS_Store',]
 
 IGNORE_FILENAME = ".bzrignore"
 
+import os
 import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
 __author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.1pre'
+__version__ = '0.0.8'
 
 
 def get_bzr_revision():
@@ -47,8 +49,7 @@
     from bzrlib.branch import Branch
     
     try:
-        import os
-        branch = Branch(os.path.dirname(__path__[0]))
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-09-15 21:35:53 +0000
+++ bzrlib/add.py	2005-09-23 09:25:16 +0000
@@ -22,7 +22,6 @@
 
 def glob_expand_for_win32(file_list):
     import glob
-    
     expanded_file_list = []
     for possible_glob in file_list:
         glob_files = glob.glob(possible_glob)
@@ -64,7 +63,7 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch(file_list[0], find_root=True)
+    b = Branch.open_containing(file_list[0])
     return smart_add_branch(b, file_list, recurse, reporter)
 
         
@@ -112,7 +111,7 @@
 
         if kind == 'directory':
             try:
-                sub_branch = Branch(af, find_root=False)
+                sub_branch = Branch.open(af)
                 sub_tree = True
             except NotBranchError:
                 sub_tree = False

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-05-26 16:09:45 +0000
+++ bzrlib/atomicfile.py	2005-09-23 09:25:16 +0000
@@ -17,7 +17,7 @@
 
 
 from warnings import warn
-
+from osutils import rename
 
 class AtomicFile(object):
     """A file that does an atomic-rename to move into place.
@@ -65,16 +65,7 @@
         self.f.close()
         self.f = None
         
-        if sys.platform == 'win32':
-            # windows cannot rename over an existing file
-            try:
-                os.remove(self.realfilename)
-            except OSError, e:
-                import errno
-                if e.errno != errno.ENOENT:
-                    raise
-                
-        os.rename(self.tmpfilename, self.realfilename)
+        rename(self.tmpfilename, self.realfilename)
 
 
     def abort(self):

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-09-15 21:35:53 +0000
+++ bzrlib/branch.py	2005-09-26 08:56:15 +0000
@@ -21,8 +21,7 @@
 import bzrlib
 from bzrlib.trace import mutter, note
 from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, \
-     splitpath, \
-     sha_file, appendpath, file_kind
+     rename, splitpath, sha_file, appendpath, file_kind
 
 from bzrlib.errors import BzrError, InvalidRevisionNumber, InvalidRevisionId, \
      DivergedBranches, NotBranchError
@@ -43,25 +42,10 @@
 # repeatedly to calculate deltas.  We could perhaps have a weakref
 # cache in memory to make this faster.
 
-# TODO: please move the revision-string syntax stuff out of the branch
-# object; it's clutter
-
-
-def find_branch(f, **args):
-    from bzrlib.transport import transport
-    from bzrlib.local_transport import LocalTransport
-    t = transport(f)
-    # FIXME: This is a hack around transport so that
-    #        We can search the local directories for
-    #        a branch root.
-    if args.has_key('init') and args['init']:
-        # Don't search if we are init-ing
-        return Branch(t, **args)
-    if isinstance(t, LocalTransport):
-        root = find_branch_root(f)
-        if root != f:
-            t = transport(root)
-    return Branch(t, **args)
+def find_branch(*ignored, **ignored_too):
+    # XXX: leave this here for about one release, then remove it
+    raise NotImplementedError('find_branch() is not supported anymore, '
+                              'please use one of the new branch constructors')
 
 def _relpath(base, path):
     """Return path relative to base, or raise exception.
@@ -102,8 +86,10 @@
     """
     if f == None:
         f = os.getcwd()
-    else:
+    elif hasattr(os.path, 'realpath'):
         f = os.path.realpath(f)
+    else:
+        f = os.path.abspath(f)
     if not os.path.exists(f):
         raise BzrError('%r does not exist' % f)
         
@@ -129,7 +115,55 @@
     """Branch holding a history of revisions.
 
     base
-        Base directory of the branch.
+        Base directory/url of the branch.
+    """
+    base = None
+
+    def __init__(self, *ignored, **ignored_too):
+        raise NotImplementedError('The Branch class is abstract')
+
+    @staticmethod
+    def open(base):
+        """Open an existing branch, rooted at 'base' (url)"""
+        if base and (base.startswith('http://') or base.startswith('https://')):
+            from bzrlib.remotebranch import RemoteBranch
+            return RemoteBranch(base, find_root=False)
+        else:
+            return LocalBranch(base, find_root=False)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        This probes for a branch at url, and searches upwards from there.
+        """
+        if url and (url.startswith('http://') or url.startswith('https://')):
+            from bzrlib.remotebranch import RemoteBranch
+            return RemoteBranch(url)
+        else:
+            return LocalBranch(url)
+
+    @staticmethod
+    def initialize(base):
+        """Create a new branch, rooted at 'base' (url)"""
+        if base and (base.startswith('http://') or base.startswith('https://')):
+            from bzrlib.remotebranch import RemoteBranch
+            return RemoteBranch(base, init=True)
+        else:
+            return LocalBranch(base, init=True)
+
+    def setup_caching(self, cache_root):
+        """Subclasses that care about caching should override this, and set
+        up cached stores located under cache_root.
+        """
+
+
+class LocalBranch(Branch):
+    """A branch stored in the actual filesystem.
+
+    Note that it's "local" in the context of the filesystem; it doesn't
+    really matter if it's on an nfs/smb/afs/coda/... share, as long as
+    it's writable, and can be accessed via the normal filesystem API.
 
     _lock_mode
         None, or 'r' or 'w'
@@ -141,43 +175,52 @@
     _lock
         Lock object from bzrlib.lock.
     """
-    base = None
+    # We actually expect this class to be somewhat short-lived; part of its
+    # purpose is to try to isolate what bits of the branch logic are tied to
+    # filesystem access, so that in a later step, we can extricate them to
+    # a separarte ("storage") class.
     _lock_mode = None
     _lock_count = None
     _lock = None
-    cache_root = None
-    
-    # Map some sort of prefix into a namespace
-    # stuff like "revno:10", "revid:", etc.
-    # This should match a prefix with a function which accepts
-    REVISION_NAMESPACES = {}
 
-    def __init__(self, transport, init=False):
+    def __init__(self, base, init=False, find_root=True):
         """Create new branch object at a particular location.
 
-        transport -- A Transport object, defining how to access files.
-                (If a string, transport.transport() will be used to
-                create a Transport object)
+        base -- Base directory for the branch. May be a file:// url.
         
         init -- If True, create new control files in a previously
              unversioned directory.  If False, the branch must already
              be versioned.
 
+        find_root -- If true and init is false, find the root of the
+             existing branch containing base.
+
         In the test suite, creation of new trees is tested using the
         `ScratchBranch` class.
         """
-        if isinstance(transport, basestring):
-            from transport import transport as get_transport
-            transport = get_transport(transport)
-
-        self._transport = transport
+        from bzrlib.store import ImmutableStore
         if init:
+            self.base = os.path.realpath(base)
             self._make_control()
+        elif find_root:
+            self.base = find_branch_root(base)
+        else:
+            if base.startswith("file://"):
+                base = base[7:]
+            self.base = os.path.realpath(base)
+            if not isdir(self.controlfilename('.')):
+                raise NotBranchError("not a bzr branch: %s" % quotefn(base),
+                                     ['use "bzr init" to initialize a new working tree',
+                                      'current bzr can only operate from top-of-tree'])
         self._check_format()
 
+        self.text_store = ImmutableStore(self.controlfilename('text-store'))
+        self.revision_store = ImmutableStore(self.controlfilename('revision-store'))
+        self.inventory_store = ImmutableStore(self.controlfilename('inventory-store'))
+
 
     def __str__(self):
-        return '%s(%r)' % (self.__class__.__name__, self._transport.base)
+        return '%s(%r)' % (self.__class__.__name__, self.base)
 
 
     __repr__ = __str__
@@ -189,33 +232,7 @@
             warn("branch %r was not explicitly unlocked" % self)
             self._lock.unlock()
 
-        # TODO: It might be best to do this somewhere else,
-        # but it is nice for a Branch object to automatically
-        # cache it's information.
-        # Alternatively, we could have the Transport objects cache requests
-        # See the earlier discussion about how major objects (like Branch)
-        # should never expect their __del__ function to run.
-        if self.cache_root is not None:
-            #from warnings import warn
-            #warn("branch %r auto-cleanup of cache files" % self)
-            try:
-                import shutil
-                shutil.rmtree(self.cache_root)
-            except:
-                pass
-            self.cache_root = None
-
-    def _get_base(self):
-        if self._transport:
-            return self._transport.base
-        return None
-
-    base = property(_get_base)
-
-
     def lock_write(self):
-        # TODO: Upgrade locking to support using a Transport,
-        # and potentially a remote locking protocol
         if self._lock_mode:
             if self._lock_mode != 'w':
                 from bzrlib.errors import LockError
@@ -223,8 +240,9 @@
                                 self._lock_mode)
             self._lock_count += 1
         else:
-            self._lock = self._transport.lock_write(
-                    self._rel_controlfilename('branch-lock'))
+            from bzrlib.lock import WriteLock
+
+            self._lock = WriteLock(self.controlfilename('branch-lock'))
             self._lock_mode = 'w'
             self._lock_count = 1
 
@@ -235,8 +253,9 @@
                    "invalid lock mode %r" % self._lock_mode
             self._lock_count += 1
         else:
-            self._lock = self._transport.lock_read(
-                    self._rel_controlfilename('branch-lock'))
+            from bzrlib.lock import ReadLock
+
+            self._lock = ReadLock(self.controlfilename('branch-lock'))
             self._lock_mode = 'r'
             self._lock_count = 1
                         
@@ -254,23 +273,19 @@
 
     def abspath(self, name):
         """Return absolute filename for something in the branch"""
-        return self._transport.abspath(name)
+        return os.path.join(self.base, name)
 
     def relpath(self, path):
         """Return path relative to this branch of something inside it.
 
         Raises an error if path is not in this branch."""
-        return self._transport.relpath(path)
-
-
-    def _rel_controlfilename(self, file_or_path):
+        return _relpath(self.base, path)
+
+    def controlfilename(self, file_or_path):
+        """Return location relative to branch."""
         if isinstance(file_or_path, basestring):
             file_or_path = [file_or_path]
-        return [bzrlib.BZRDIR] + file_or_path
-
-    def controlfilename(self, file_or_path):
-        """Return location relative to branch."""
-        return self._transport.abspath(self._rel_controlfilename(file_or_path))
+        return os.path.join(self.base, bzrlib.BZRDIR, *file_or_path)
 
 
     def controlfile(self, file_or_path, mode='r'):
@@ -284,78 +299,43 @@
         Controlfiles should almost never be opened in write mode but
         rather should be atomically copied and replaced using atomicfile.
         """
-        import codecs
-
-        relpath = self._rel_controlfilename(file_or_path)
-        #TODO: codecs.open() buffers linewise, so it was overloaded with
-        # a much larger buffer, do we need to do the same for getreader/getwriter?
-        if mode == 'rb': 
-            return self._transport.get(relpath)
-        elif mode == 'wb':
-            raise BzrError("Branch.controlfile(mode='wb') is not supported, use put_controlfiles")
-        elif mode == 'r':
-            return codecs.getreader('utf-8')(self._transport.get(relpath), errors='replace')
-        elif mode == 'w':
-            raise BzrError("Branch.controlfile(mode='w') is not supported, use put_controlfiles")
+
+        fn = self.controlfilename(file_or_path)
+
+        if mode == 'rb' or mode == 'wb':
+            return file(fn, mode)
+        elif mode == 'r' or mode == 'w':
+            # open in binary mode anyhow so there's no newline translation;
+            # codecs uses line buffering by default; don't want that.
+            import codecs
+            return codecs.open(fn, mode + 'b', 'utf-8',
+                               buffering=60000)
         else:
             raise BzrError("invalid controlfile mode %r" % mode)
 
-    def put_controlfile(self, path, f, encode=True):
-        """Write an entry as a controlfile.
-
-        :param path: The path to put the file, relative to the .bzr control
-                     directory
-        :param f: A file-like or string object whose contents should be copied.
-        :param encode:  If true, encode the contents as utf-8
-        """
-        self.put_controlfiles([(path, f)], encode=encode)
-
-    def put_controlfiles(self, files, encode=True):
-        """Write several entries as controlfiles.
-
-        :param files: A list of [(path, file)] pairs, where the path is the directory
-                      underneath the bzr control directory
-        :param encode:  If true, encode the contents as utf-8
-        """
-        import codecs
-        ctrl_files = []
-        for path, f in files:
-            if encode:
-                if isinstance(f, basestring):
-                    f = f.encode('utf-8', 'replace')
-                else:
-                    f = codecs.getwriter('utf-8')(f, errors='replace')
-            path = self._rel_controlfilename(path)
-            ctrl_files.append((path, f))
-        self._transport.put_multi(ctrl_files)
-
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from cStringIO import StringIO
         
-        # Create an empty inventory
-        sio = StringIO()
+        os.mkdir(self.controlfilename([]))
+        self.controlfile('README', 'w').write(
+            "This is a Bazaar-NG control directory.\n"
+            "Do not change any files in this directory.\n")
+        self.controlfile('branch-format', 'w').write(BZR_BRANCH_FORMAT)
+        for d in ('text-store', 'inventory-store', 'revision-store'):
+            os.mkdir(self.controlfilename(d))
+        for f in ('revision-history', 'merged-patches',
+                  'pending-merged-patches', 'branch-name',
+                  'branch-lock',
+                  'pending-merges'):
+            self.controlfile(f, 'w').write('')
+        mutter('created control directory in ' + self.base)
+
         # if we want per-tree root ids then this is the place to set
         # them; they're not needed for now and so ommitted for
         # simplicity.
-        bzrlib.xml.serializer_v4.write_inventory(Inventory(), sio)
+        f = self.controlfile('inventory','w')
+        bzrlib.xml.serializer_v4.write_inventory(Inventory(), f)
 
-        dirs = [[], 'text-store', 'inventory-store', 'revision-store']
-        files = [('README', 
-            "This is a Bazaar-NG control directory.\n"
-            "Do not change any files in this directory.\n"),
-            ('branch-format', BZR_BRANCH_FORMAT),
-            ('revision-history', ''),
-            ('merged-patches', ''),
-            ('pending-merged-patches', ''),
-            ('branch-name', ''),
-            ('branch-lock', ''),
-            ('pending-merges', ''),
-            ('inventory', sio.getvalue())
-        ]
-        self._transport.mkdir_multi([self._rel_controlfilename(d) for d in dirs])
-        self.put_controlfiles(files)
-        mutter('created control directory in ' + self._transport.base)
 
     def _check_format(self):
         """Check this branch format is supported.
@@ -375,31 +355,6 @@
                            ['use a different bzr version',
                             'or remove the .bzr directory and "bzr init" again'])
 
-        # We know that the format is the currently supported one.
-        # So create the rest of the entries.
-        from bzrlib.store.compressed_text import CompressedTextStore
-
-        if self._transport.should_cache():
-            import tempfile
-            self.cache_root = tempfile.mkdtemp(prefix='bzr-cache')
-            mutter('Branch %r using caching in %r' % (self, self.cache_root))
-        else:
-            self.cache_root = None
-
-        def get_store(name):
-            relpath = self._rel_controlfilename(name)
-            store = CompressedTextStore(self._transport.clone(relpath))
-            if self._transport.should_cache():
-                from meta_store import CachedStore
-                cache_path = os.path.join(self.cache_root, name)
-                os.mkdir(cache_path)
-                store = CachedStore(store, cache_path)
-            return store
-
-        self.text_store = get_store('text-store')
-        self.revision_store = get_store('revision-store')
-        self.inventory_store = get_store('inventory-store')
-
     def get_root_id(self):
         """Return the id of this branches root"""
         inv = self.read_working_inventory()
@@ -436,14 +391,16 @@
         That is to say, the inventory describing changes underway, that
         will be committed to the next revision.
         """
-        from cStringIO import StringIO
+        from bzrlib.atomicfile import AtomicFile
+        
         self.lock_write()
         try:
-            sio = StringIO()
-            bzrlib.xml.serializer_v4.write_inventory(inv, sio)
-            sio.seek(0)
-            # Transport handles atomicity
-            self.put_controlfile('inventory', sio)
+            f = AtomicFile(self.controlfilename('inventory'), 'wb')
+            try:
+                bzrlib.xml.serializer_v4.write_inventory(inv, f)
+                f.commit()
+            finally:
+                f.close()
         finally:
             self.unlock()
         
@@ -527,7 +484,7 @@
         """Print `file` to stdout."""
         self.lock_read()
         try:
-            tree = self.revision_tree(self.lookup_revision(revno))
+            tree = self.revision_tree(self.get_rev_id(revno))
             # use inventory as it was in that revision
             file_id = tree.inventory.path2id(file)
             if not file_id:
@@ -614,17 +571,21 @@
 
 
     def append_revision(self, *revision_ids):
+        from bzrlib.atomicfile import AtomicFile
+
         for revision_id in revision_ids:
             mutter("add {%s} to revision-history" % revision_id)
 
         rev_history = self.revision_history()
         rev_history.extend(revision_ids)
 
-        self.lock_write()
+        f = AtomicFile(self.controlfilename('revision-history'))
         try:
-            self.put_controlfile('revision-history', '\n'.join(rev_history))
+            for rev_id in rev_history:
+                print >>f, rev_id
+            f.commit()
         finally:
-            self.unlock()
+            f.close()
 
 
     def get_revision_xml_file(self, revision_id):
@@ -683,34 +644,7 @@
         return compare_trees(old_tree, new_tree)
 
         
-    def get_revisions(self, revision_ids, pb=None):
-        """Return the Revision object for a set of named revisions"""
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
 
-        # TODO: We need to decide what to do here
-        # we cannot use a generator with a try/finally, because
-        # you cannot guarantee that the caller will iterate through
-        # all entries.
-        # in the past, get_inventory wasn't even wrapped in a
-        # try/finally locking block.
-        # We could either lock without the try/finally, or just
-        # not lock at all. We are reading entries that should
-        # never be updated.
-        # I prefer locking with no finally, so that if someone
-        # asks for a list of revisions, but doesn't consume them,
-        # that is their problem, and they will suffer the consequences
-        self.lock_read()
-        for xml_file in self.revision_store.get(revision_ids, pb=pb):
-            try:
-                r = bzrlib.xml.serializer_v4.read_revision(xml_file)
-            except SyntaxError, e:
-                raise bzrlib.errors.BzrError('failed to unpack revision_xml',
-                                             [revision_id,
-                                              str(e)])
-            yield r
-        self.unlock()
-            
     def get_revision_sha1(self, revision_id):
         """Hash the stored value of a revision, and return it."""
         # In the future, revision entries will be signed. At that
@@ -727,39 +661,19 @@
 
         TODO: Perhaps for this and similar methods, take a revision
                parameter which can be either an integer revno or a
-               string hash.
-        """
+               string hash."""
+        from bzrlib.inventory import Inventory
+
         f = self.get_inventory_xml_file(inventory_id)
         return bzrlib.xml.serializer_v4.read_inventory(f)
 
 
     def get_inventory_xml(self, inventory_id):
         """Get inventory XML as a file object."""
-        # Shouldn't this have a read-lock around it?
-        # As well as some sort of trap for missing ids?
         return self.inventory_store[inventory_id]
 
     get_inventory_xml_file = get_inventory_xml
             
-    def get_inventories(self, inventory_ids, pb=None, ignore_missing=False):
-        """Get Inventory objects by id
-        """
-        from bzrlib.inventory import Inventory
-
-        # See the discussion in get_revisions for why
-        # we don't use a try/finally block here
-        self.lock_read()
-        for f in self.inventory_store.get(inventory_ids, pb=pb, ignore_missing=ignore_missing):
-            if f is not None:
-                # TODO: Possibly put a try/except around this to handle
-                # read serialization errors
-                r = bzrlib.xml.serializer_v4.read_inventory(f)
-                yield r
-            elif ignore_missing:
-                yield None
-            else:
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        self.unlock()
 
     def get_inventory_sha1(self, inventory_id):
         """Return the sha1 hash of the inventory entry
@@ -911,7 +825,7 @@
         if stop_revision is None:
             other_revision = other.last_patch()
         else:
-            other_revision = other.lookup_revision(stop_revision)
+            other_revision = other.get_rev_id(stop_revision)
         count = greedy_fetch(self, other, other_revision, pb)[0]
         try:
             revision_ids = self.missing_revisions(other, stop_revision)
@@ -927,63 +841,55 @@
         pb.clear()
 
     def install_revisions(self, other, revision_ids, pb):
-        # We are going to iterate this many times, so make sure
-        # that it is a list, and not a generator
-        revision_ids = list(revision_ids)
         if hasattr(other.revision_store, "prefetch"):
             other.revision_store.prefetch(revision_ids)
         if hasattr(other.inventory_store, "prefetch"):
+            inventory_ids = []
+            for rev_id in revision_ids:
+                try:
+                    revision = other.get_revision(rev_id).inventory_id
+                    inventory_ids.append(revision)
+                except bzrlib.errors.NoSuchRevision:
+                    pass
             other.inventory_store.prefetch(inventory_ids)
 
         if pb is None:
             pb = bzrlib.ui.ui_factory.progress_bar()
                 
-        # This entire next section is generally done
-        # with either generators, or bulk updates
-        inventories = other.get_inventories(revision_ids, ignore_missing=True)
+        revisions = []
         needed_texts = set()
+        i = 0
 
         failures = set()
-        good_revisions = set()
-        for i, (inv, rev_id) in enumerate(zip(inventories, revision_ids)):
+        for i, rev_id in enumerate(revision_ids):
             pb.update('fetching revision', i+1, len(revision_ids))
-
-            # We don't really need to get the revision here, because
-            # the only thing we needed was the inventory_id, which now
-            # is (by design) identical to the revision_id
-            # try:
-            #     rev = other.get_revision(rev_id)
-            # except bzrlib.errors.NoSuchRevision:
-            #     failures.add(rev_id)
-            #     continue
-
-            if inv is None:
+            try:
+                rev = other.get_revision(rev_id)
+            except bzrlib.errors.NoSuchRevision:
                 failures.add(rev_id)
                 continue
-            else:
-                good_revisions.add(rev_id)
 
-            text_ids = []
+            revisions.append(rev)
+            inv = other.get_inventory(str(rev.inventory_id))
             for key, entry in inv.iter_entries():
                 if entry.text_id is None:
                     continue
-                text_ids.append(entry.text_id)
-
-            has_ids = self.text_store.has(text_ids)
-            for has, text_id in zip(has_ids, text_ids):
-                if not has:
-                    needed_texts.add(text_id)
+                if entry.text_id not in self.text_store:
+                    needed_texts.add(entry.text_id)
 
         pb.clear()
                     
         count, cp_fail = self.text_store.copy_multi(other.text_store, 
                                                     needed_texts)
         #print "Added %d texts." % count 
-        count, cp_fail = self.inventory_store.copy_multi(other.inventory_store,
-                                                         good_revisions)
+        inventory_ids = [ f.inventory_id for f in revisions ]
+        count, cp_fail = self.inventory_store.copy_multi(other.inventory_store, 
+                                                         inventory_ids)
         #print "Added %d inventories." % count 
+        revision_ids = [ f.revision_id for f in revisions]
+
         count, cp_fail = self.revision_store.copy_multi(other.revision_store, 
-                                                          good_revisions,
+                                                          revision_ids,
                                                           permit_failure=True)
         assert len(cp_fail) == 0 
         return count, failures
@@ -993,13 +899,6 @@
         from bzrlib.commit import commit
         commit(self, *args, **kw)
         
-
-    def lookup_revision(self, revision):
-        """Return the revision identifier for a given revision information."""
-        revno, info = self._get_revision_info(revision)
-        return info
-
-
     def revision_id_to_revno(self, revision_id):
         """Given a revision id, return its revno"""
         history = self.revision_history()
@@ -1008,20 +907,6 @@
         except ValueError:
             raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
-
-    def get_revision_info(self, revision):
-        """Return (revno, revision id) for revision identifier.
-
-        revision can be an integer, in which case it is assumed to be revno (though
-            this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something like
-            'date:' or 'revid:' etc.
-        """
-        revno, rev_id = self._get_revision_info(revision)
-        if revno is None:
-            raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
     def get_rev_id(self, revno, history=None):
         """Find the revision id of the specified revno."""
         if revno == 0:
@@ -1032,191 +917,6 @@
             raise bzrlib.errors.NoSuchRevision(self, revno)
         return history[revno - 1]
 
-    def _get_revision_info(self, revision):
-        """Return (revno, revision id) for revision specifier.
-
-        revision can be an integer, in which case it is assumed to be revno
-        (though this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something
-        like 'date:' or 'revid:' etc.
-
-        A revid is always returned.  If it is None, the specifier referred to
-        the null revision.  If the revid does not occur in the revision
-        history, revno will be None.
-        """
-        
-        if revision is None:
-            return 0, None
-        revno = None
-        try:# Convert to int if possible
-            revision = int(revision)
-        except ValueError:
-            pass
-        revs = self.revision_history()
-        if isinstance(revision, int):
-            if revision < 0:
-                revno = len(revs) + revision + 1
-            else:
-                revno = revision
-            rev_id = self.get_rev_id(revno, revs)
-        elif isinstance(revision, basestring):
-            for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
-                if revision.startswith(prefix):
-                    result = func(self, revs, revision)
-                    if len(result) > 1:
-                        revno, rev_id = result
-                    else:
-                        revno = result[0]
-                        rev_id = self.get_rev_id(revno, revs)
-                    break
-            else:
-                raise BzrError('No namespace registered for string: %r' %
-                               revision)
-        else:
-            raise TypeError('Unhandled revision type %s' % revision)
-
-        if revno is None:
-            if rev_id is None:
-                raise bzrlib.errors.NoSuchRevision(self, revision)
-        return revno, rev_id
-
-    def _namespace_revno(self, revs, revision):
-        """Lookup a revision by revision number"""
-        assert revision.startswith('revno:')
-        try:
-            return (int(revision[6:]),)
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revno:'] = _namespace_revno
-
-    def _namespace_revid(self, revs, revision):
-        assert revision.startswith('revid:')
-        rev_id = revision[len('revid:'):]
-        try:
-            return revs.index(rev_id) + 1, rev_id
-        except ValueError:
-            return None, rev_id
-    REVISION_NAMESPACES['revid:'] = _namespace_revid
-
-    def _namespace_last(self, revs, revision):
-        assert revision.startswith('last:')
-        try:
-            offset = int(revision[5:])
-        except ValueError:
-            return (None,)
-        else:
-            if offset <= 0:
-                raise BzrError('You must supply a positive value for --revision last:XXX')
-            return (len(revs) - offset + 1,)
-    REVISION_NAMESPACES['last:'] = _namespace_last
-
-    def _namespace_tag(self, revs, revision):
-        assert revision.startswith('tag:')
-        raise BzrError('tag: namespace registered, but not implemented.')
-    REVISION_NAMESPACES['tag:'] = _namespace_tag
-
-    def _namespace_date(self, revs, revision):
-        assert revision.startswith('date:')
-        import datetime
-        # Spec for date revisions:
-        #   date:value
-        #   value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
-        #   it can also start with a '+/-/='. '+' says match the first
-        #   entry after the given date. '-' is match the first entry before the date
-        #   '=' is match the first entry after, but still on the given date.
-        #
-        #   +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
-        #   -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
-        #   =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
-        #       May 13th, 2005 at 0:00
-        #
-        #   So the proper way of saying 'give me all entries for today' is:
-        #       -r {date:+today}:{date:-tomorrow}
-        #   The default is '=' when not supplied
-        val = revision[5:]
-        match_style = '='
-        if val[:1] in ('+', '-', '='):
-            match_style = val[:1]
-            val = val[1:]
-
-        today = datetime.datetime.today().replace(hour=0,minute=0,second=0,microsecond=0)
-        if val.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif val.lower() == 'today':
-            dt = today
-        elif val.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
-        else:
-            import re
-            # This should be done outside the function to avoid recompiling it.
-            _date_re = re.compile(
-                    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-                    r'(,|T)?\s*'
-                    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-                )
-            m = _date_re.match(val)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise BzrError('Invalid revision date %r' % revision)
-
-            if m.group('date'):
-                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
-            else:
-                year, month, day = today.year, today.month, today.day
-            if m.group('time'):
-                hour = int(m.group('hour'))
-                minute = int(m.group('minute'))
-                if m.group('second'):
-                    second = int(m.group('second'))
-                else:
-                    second = 0
-            else:
-                hour, minute, second = 0,0,0
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                    hour=hour, minute=minute, second=second)
-        first = dt
-        last = None
-        reversed = False
-        if match_style == '-':
-            reversed = True
-        elif match_style == '=':
-            last = dt + datetime.timedelta(days=1)
-
-        if reversed:
-            for i in range(len(revs)-1, -1, -1):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first >= dt and (last is None or dt >= last):
-                    return (i+1,)
-        else:
-            for i in range(len(revs)):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first <= dt and (last is None or dt <= last):
-                    return (i+1,)
-    REVISION_NAMESPACES['date:'] = _namespace_date
-
-
-    def _namespace_ancestor(self, revs, revision):
-        from revision import common_ancestor, MultipleRevisionSources
-        other_branch = find_branch(_trim_namespace('ancestor', revision))
-        revision_a = self.last_patch()
-        revision_b = other_branch.last_patch()
-        for r, b in ((revision_a, self), (revision_b, other_branch)):
-            if r is None:
-                raise bzrlib.errors.NoCommits(b)
-        revision_source = MultipleRevisionSources(self, other_branch)
-        result = common_ancestor(revision_a, revision_b, revision_source)
-        try:
-            revno = self.revision_id_to_revno(result)
-        except bzrlib.errors.NoSuchRevision:
-            revno = None
-        return revno,result
-        
-
-    REVISION_NAMESPACES['ancestor:'] = _namespace_ancestor
 
     def revision_tree(self, revision_id):
         """Return Tree for a revision on this branch.
@@ -1235,8 +935,7 @@
     def working_tree(self):
         """Return a `Tree` for the working copy."""
         from bzrlib.workingtree import WorkingTree
-        # TODO: In the future, WorkingTree should utilize Transport
-        return WorkingTree(self._transport.base, self.read_working_inventory())
+        return WorkingTree(self.base, self.read_working_inventory())
 
 
     def basis_tree(self):
@@ -1290,7 +989,7 @@
             from_abs = self.abspath(from_rel)
             to_abs = self.abspath(to_rel)
             try:
-                os.rename(from_abs, to_abs)
+                rename(from_abs, to_abs)
             except OSError, e:
                 raise BzrError("failed to rename %r to %r: %s"
                         % (from_abs, to_abs, e[1]),
@@ -1359,7 +1058,7 @@
                 result.append((f, dest_path))
                 inv.rename(inv.path2id(f), to_dir_id, name_tail)
                 try:
-                    os.rename(self.abspath(f), self.abspath(dest_path))
+                    rename(self.abspath(f), self.abspath(dest_path))
                 except OSError, e:
                     raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
                             ["rename rolled back"])
@@ -1421,8 +1120,8 @@
         These are revisions that have been merged into the working
         directory but not yet committed.
         """
-        cfn = self._rel_controlfilename('pending-merges')
-        if not self._transport.has(cfn):
+        cfn = self.controlfilename('pending-merges')
+        if not os.path.exists(cfn):
             return []
         p = []
         for l in self.controlfile('pending-merges', 'r').readlines():
@@ -1430,26 +1129,29 @@
         return p
 
 
-    def add_pending_merge(self, *revision_ids):
+    def add_pending_merge(self, revision_id):
         from bzrlib.revision import validate_revision_id
 
-        for rev_id in revision_ids:
-            validate_revision_id(rev_id)
+        validate_revision_id(revision_id)
 
         p = self.pending_merges()
-        updated = False
-        for rev_id in revision_ids:
-            if rev_id in p:
-                continue
-            p.append(rev_id)
-            updated = True
-        if updated:
-            self.set_pending_merges(p)
+        if revision_id in p:
+            return
+        p.append(revision_id)
+        self.set_pending_merges(p)
+
 
     def set_pending_merges(self, rev_list):
+        from bzrlib.atomicfile import AtomicFile
         self.lock_write()
         try:
-            self.put_controlfile('pending-merges', '\n'.join(rev_list))
+            f = AtomicFile(self.controlfilename('pending-merges'))
+            try:
+                for l in rev_list:
+                    print >>f, l
+                f.commit()
+            finally:
+                f.close()
         finally:
             self.unlock()
 
@@ -1503,9 +1205,10 @@
             raise InvalidRevisionNumber(revno)
         
         
-
-
-class ScratchBranch(Branch):
+        
+
+
+class ScratchBranch(LocalBranch):
     """Special test class: a branch that cleans up after itself.
 
     >>> b = ScratchBranch()
@@ -1528,19 +1231,23 @@
         if base is None:
             base = mkdtemp()
             init = True
-        Branch.__init__(self, base, init=init)
+        LocalBranch.__init__(self, base, init=init)
         for d in dirs:
-            self._transport.mkdir(d)
+            os.mkdir(self.abspath(d))
             
         for f in files:
-            self._transport.put(f, 'content of %s' % f)
+            file(os.path.join(self.base, f), 'w').write('content of %s' % f)
 
 
     def clone(self):
         """
         >>> orig = ScratchBranch(files=["file1", "file2"])
         >>> clone = orig.clone()
-        >>> os.path.samefile(orig.base, clone.base)
+        >>> if os.name != 'nt':
+        ...   os.path.samefile(orig.base, clone.base)
+        ... else:
+        ...   orig.base == clone.base
+        ...
         False
         >>> os.path.isfile(os.path.join(clone.base, "file1"))
         True
@@ -1572,7 +1279,7 @@
                 for name in files:
                     os.chmod(os.path.join(root, name), 0700)
             rmtree(self.base)
-        self._transport = None
+        self.base = None
 
     
 
@@ -1629,7 +1336,7 @@
     return gen_file_id('TREE_ROOT')
 
 
-def copy_branch(branch_from, to_location, revision=None):
+def copy_branch(branch_from, to_location, revno=None):
     """Copy branch_from into the existing directory to_location.
 
     revision
@@ -1644,19 +1351,12 @@
     assert isinstance(branch_from, Branch)
     assert isinstance(to_location, basestring)
     
-    br_to = Branch(to_location, init=True)
+    br_to = Branch.initialize(to_location)
     br_to.set_root_id(branch_from.get_root_id())
-    if revision is None:
+    if revno is None:
         revno = branch_from.revno()
-    else:
-        revno, rev_id = branch_from.get_revision_info(revision)
     br_to.update_revisions(branch_from, stop_revision=revno)
     merge((to_location, -1), (to_location, 0), this_dir=to_location,
           check_clean=False, ignore_zero=True)
     br_to.set_parent(branch_from.base)
     return br_to
-
-def _trim_namespace(namespace, spec):
-    full_namespace = namespace + ':'
-    assert spec.startswith(full_namespace)
-    return spec[len(full_namespace):]

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-09-14 17:23:14 +0000
+++ bzrlib/builtins.py	2005-09-23 03:22:16 +0000
@@ -22,7 +22,7 @@
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
-from bzrlib.branch import find_branch
+from bzrlib.branch import Branch
 from bzrlib import BZRDIR
 from bzrlib.commands import Command
 
@@ -62,47 +62,68 @@
     directory is shown.  Otherwise, only the status of the specified
     files or directories is reported.  If a directory is given, status
     is reported for everything inside that directory.
+
+    If a revision argument is given, the status is calculated against
+    that revision, or between two revisions if two are provided.
     """
     # XXX: FIXME: bzr status should accept a -r option to show changes
     # relative to a revision, or between revisions
 
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids']
+    takes_options = ['all', 'show-ids', 'revision']
     aliases = ['st', 'stat']
     
-    def run(self, all=False, show_ids=False, file_list=None):
+    def run(self, all=False, show_ids=False, file_list=None, revision=None):
         if file_list:
-            b = find_branch(file_list[0])
+            b = Branch.open_containing(file_list[0])
             file_list = [b.relpath(x) for x in file_list]
             # special case: only one path was given and it's the root
             # of the branch
             if file_list == ['']:
                 file_list = None
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
             
         from bzrlib.status import show_status
         show_status(b, show_unchanged=all, show_ids=show_ids,
-                    specific_files=file_list)
+                    specific_files=file_list, revision=revision)
 
 
 class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
+    """Write out metadata for a revision.
+    
+    The revision to print can either be specified by a specific
+    revision identifier, or you can use --revision.
+    """
 
     hidden = True
-    takes_args = ['revision_id']
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
     
-    def run(self, revision_id):
-        b = find_branch('.')
-        sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+    def run(self, revision_id=None, revision=None):
+        from bzrlib.revisionspec import RevisionSpec
 
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = Branch.open_containing('.')
+        if revision_id is not None:
+            sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+        elif revision is not None:
+            for rev in revision:
+                if rev is None:
+                    raise BzrCommandError('You cannot specify a NULL revision.')
+                revno, rev_id = rev.in_history(b)
+                sys.stdout.write(b.get_revision_xml_file(rev_id).read())
+    
 
 class cmd_revno(Command):
     """Show current revision number.
 
     This is equal to the number of revisions on this branch."""
     def run(self):
-        print find_branch('.').revno()
+        print Branch.open_containing('.').revno()
 
 
 class cmd_revision_info(Command):
@@ -111,21 +132,26 @@
     hidden = True
     takes_args = ['revision_info*']
     takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
+    def run(self, revision=None, revision_info_list=[]):
+        from bzrlib.revisionspec import RevisionSpec
 
         revs = []
         if revision is not None:
             revs.extend(revision)
         if revision_info_list is not None:
-            revs.extend(revision_info_list)
+            for rev in revision_info_list:
+                revs.append(RevisionSpec(rev))
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
 
         for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
+            revinfo = rev.in_history(b)
+            if revinfo.revno is None:
+                print '     %s' % revinfo.rev_id
+            else:
+                print '%4d %s' % (revinfo.revno, revinfo.rev_id)
 
     
 class cmd_add(Command):
@@ -174,7 +200,7 @@
         for d in dir_list:
             os.mkdir(d)
             if not b:
-                b = find_branch(d)
+                b = Branch.open_containing(d)
             b.add([d])
             print 'added', d
 
@@ -185,7 +211,7 @@
     hidden = True
     
     def run(self, filename):
-        print find_branch(filename).relpath(filename)
+        print Branch.open_containing(filename).relpath(filename)
 
 
 
@@ -194,14 +220,14 @@
     takes_options = ['revision', 'show-ids']
     
     def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
+        b = Branch.open_containing('.')
+        if revision is None:
             inv = b.read_working_inventory()
         else:
             if len(revision) > 1:
                 raise BzrCommandError('bzr inventory --revision takes'
                     ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
+            inv = b.get_revision_inventory(revision[0].in_history(b).rev_id)
 
         for path, entry in inv.entries():
             if show_ids:
@@ -220,7 +246,7 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
 
         # TODO: glob expansion on windows?
         b.move([b.relpath(s) for s in source_list], b.relpath(dest))
@@ -243,7 +269,7 @@
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         b.rename_one(b.relpath(from_name), b.relpath(to_name))
 
 
@@ -265,7 +291,7 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = find_branch(names_list[0])
+        b = Branch.open_containing(names_list[0])
 
         rel_names = [b.relpath(x) for x in names_list]
         
@@ -305,7 +331,7 @@
         from shutil import rmtree
         import errno
         
-        br_to = find_branch('.')
+        br_to = Branch.open_containing('.')
         stored_loc = br_to.get_parent()
         if location is None:
             if stored_loc is None:
@@ -314,13 +340,14 @@
                 print "Using last location: %s" % stored_loc
                 location = stored_loc
         cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
+        from bzrlib.errors import DivergedBranches
+        br_from = Branch.open_containing(location)
         location = br_from.base
         old_revno = br_to.revno()
         try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
+            from bzrlib.errors import DivergedBranches
+            br_from = Branch.open(location)
+            br_from.setup_caching(cache_root)
             location = br_from.base
             old_revno = br_to.revno()
             try:
@@ -351,7 +378,7 @@
     aliases = ['get', 'clone']
 
     def run(self, from_location, to_location=None, revision=None):
-        from bzrlib.branch import copy_branch, find_cached_branch
+        from bzrlib.branch import copy_branch
         import tempfile
         import errno
         from shutil import rmtree
@@ -363,13 +390,18 @@
                 raise BzrCommandError(
                     'bzr branch --revision takes exactly 1 revision value')
             try:
-                br_from = find_cached_branch(from_location, cache_root)
+                br_from = Branch.open(from_location)
             except OSError, e:
                 if e.errno == errno.ENOENT:
                     raise BzrCommandError('Source location "%s" does not'
                                           ' exist.' % to_location)
                 else:
                     raise
+            br_from.setup_caching(cache_root)
+            if len(revision) == 1 and revision[0] is not None:
+                revno = revision[0].in_history(br_from)[0]
+            else:
+                revno = None
             if to_location is None:
                 to_location = os.path.basename(from_location.rstrip("/\\"))
             try:
@@ -384,7 +416,7 @@
                 else:
                     raise
             try:
-                copy_branch(br_from, to_location, revision[0])
+                copy_branch(br_from, to_location, revno)
             except bzrlib.errors.NoSuchRevision:
                 rmtree(to_location)
                 msg = "The branch %s has no revision %d." % (from_location, revision[0])
@@ -403,7 +435,7 @@
     takes_args = ['dir?']
 
     def run(self, dir='.'):
-        b = find_branch(dir)
+        b = Branch.open_containing(dir)
         old_inv = b.basis_tree().inventory
         new_inv = b.read_working_inventory()
 
@@ -420,7 +452,7 @@
     def run(self, branch=None):
         import info
 
-        b = find_branch(branch)
+        b = Branch.open_containing(branch)
         info.show_info(b)
 
 
@@ -434,7 +466,7 @@
     takes_options = ['verbose']
     
     def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
+        b = Branch.open_containing(file_list[0])
         b.remove([b.relpath(f) for f in file_list], verbose=verbose)
 
 
@@ -448,7 +480,7 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         i = b.inventory.path2id(b.relpath(filename))
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
@@ -464,7 +496,7 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         inv = b.inventory
         fid = inv.path2id(b.relpath(filename))
         if fid == None:
@@ -477,14 +509,14 @@
     """Display list of revision ids on this branch."""
     hidden = True
     def run(self):
-        for patchid in find_branch('.').revision_history():
+        for patchid in Branch.open_containing('.').revision_history():
             print patchid
 
 
 class cmd_directories(Command):
     """Display list of versioned directories in this branch."""
     def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
+        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
             if name == '':
                 print '.'
             else:
@@ -506,7 +538,7 @@
     """
     def run(self):
         from bzrlib.branch import Branch
-        Branch('.', init=True)
+        Branch.initialize('.')
 
 
 class cmd_diff(Command):
@@ -544,13 +576,13 @@
         from bzrlib.diff import show_diff
 
         if file_list:
-            b = find_branch(file_list[0])
+            b = Branch.open_containing(file_list[0])
             file_list = [b.relpath(f) for f in file_list]
             if file_list == ['']:
                 # just pointing to top-of-tree
                 file_list = None
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
 
         if revision is not None:
             if len(revision) == 1:
@@ -575,7 +607,7 @@
     TODO: Show files deleted since a previous revision, or between two revisions.
     """
     def run(self, show_ids=False):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         old = b.basis_tree()
         new = b.working_tree()
 
@@ -598,7 +630,7 @@
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         td = compare_trees(b.basis_tree(), b.working_tree())
 
         for path, id, kind in td.modified:
@@ -610,7 +642,7 @@
     """List files added in working tree."""
     hidden = True
     def run(self):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         wt = b.working_tree()
         basis_inv = b.basis_tree().inventory
         inv = wt.inventory
@@ -632,7 +664,7 @@
     takes_args = ['filename?']
     def run(self, filename=None):
         """Print the branch root."""
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         print b.base
 
 
@@ -668,24 +700,24 @@
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = find_branch(filename)
+            b = Branch.open_containing(filename)
             fp = b.relpath(filename)
             if fp:
                 file_id = b.read_working_inventory().path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = find_branch('.')
+            b = Branch.open_containing('.')
             file_id = None
 
         if revision is None:
             rev1 = None
             rev2 = None
         elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
+            rev1 = rev2 = revision[0].in_history(b).revno
         elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
+            rev1 = revision[0].in_history(b).revno
+            rev2 = revision[1].in_history(b).revno
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 
@@ -727,7 +759,7 @@
     hidden = True
     takes_args = ["filename"]
     def run(self, filename):
-        b = find_branch(filename)
+        b = Branch.open_containing(filename)
         inv = b.read_working_inventory()
         file_id = inv.path2id(b.relpath(filename))
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
@@ -741,11 +773,11 @@
     """
     hidden = True
     def run(self, revision=None, verbose=False):
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if revision == None:
             tree = b.working_tree()
         else:
-            tree = b.revision_tree(b.lookup_revision(revision))
+            tree = b.revision_tree(revision.in_history(b).rev_id)
 
         for fp, fc, kind, fid in tree.list_files():
             if verbose:
@@ -766,7 +798,7 @@
     """List unknown files."""
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
+        for f in Branch.open_containing('.').unknowns():
             print quotefn(f)
 
 
@@ -794,7 +826,7 @@
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         ifn = b.abspath('.bzrignore')
 
         if os.path.exists(ifn):
@@ -834,7 +866,7 @@
 
     See also: bzr ignore"""
     def run(self):
-        tree = find_branch('.').working_tree()
+        tree = Branch.open_containing('.').working_tree()
         for path, file_class, kind, file_id in tree.list_files():
             if file_class != 'I':
                 continue
@@ -858,7 +890,7 @@
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print find_branch('.').lookup_revision(revno)
+        print Branch.open_containing('.').get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -877,13 +909,13 @@
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if revision is None:
             rev_id = b.last_patch()
         else:
             if len(revision) != 1:
                 raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
+            rev_id = revision[0].in_history(b).rev_id
         t = b.revision_tree(rev_id)
         root, ext = os.path.splitext(dest)
         if not format:
@@ -905,12 +937,12 @@
     takes_args = ['filename']
 
     def run(self, filename, revision=None):
-        if revision == None:
+        if revision is None:
             raise BzrCommandError("bzr cat requires a revision number")
         elif len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
+        b = Branch.open_containing('.')
+        b.print_file(b.relpath(filename), revision[0].in_history(b).revno)
 
 
 class cmd_local_time_offset(Command):
@@ -951,7 +983,7 @@
         from bzrlib.status import show_status
         from cStringIO import StringIO
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         if selected_list:
             selected_list = [b.relpath(s) for s in selected_list]
             
@@ -993,7 +1025,7 @@
     def run(self, dir='.'):
         from bzrlib.check import check
 
-        check(find_branch(dir))
+        check(Branch.open_containing(dir))
 
 
 class cmd_scan_cache(Command):
@@ -1026,7 +1058,7 @@
 
     def run(self, dir='.'):
         from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
+        upgrade(Branch.open_containing(dir))
 
 
 
@@ -1036,7 +1068,7 @@
     
     def run(self, email=False):
         try:
-            b = bzrlib.branch.find_branch('.')
+            b = bzrlib.branch.Branch.open_containing('.')
         except:
             b = None
         
@@ -1108,8 +1140,8 @@
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = find_branch(branch)
-        branch2 = find_branch(other)
+        branch1 = Branch.open_containing(branch)
+        branch2 = Branch.open_containing(other)
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
@@ -1178,15 +1210,18 @@
             other = [branch, -1]
         else:
             if len(revision) == 1:
-                other = [branch, revision[0]]
                 base = [None, None]
+                other = [branch, revision[0].in_history(branch).revno]
             else:
                 assert len(revision) == 2
                 if None in revision:
                     raise BzrCommandError(
                         "Merge doesn't permit that revision specifier.")
-                base = [branch, revision[0]]
-                other = [branch, revision[1]]
+                from bzrlib.branch import Branch
+                b = Branch.open(branch)
+
+                base = [branch, revision[0].in_history(b).revno]
+                other = [branch, revision[1].in_history(b).revno]
 
         try:
             merge(other, base, check_clean=(not force), merge_type=merge_type)
@@ -1220,16 +1255,19 @@
             if len(file_list) == 0:
                 raise BzrCommandError("No files specified")
         if revision is None:
-            revision = [-1]
+            revno = -1
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
+        else:
+            b = Branch.open_containing('.')
+            revno = revision[0].in_history(b).revno
+        merge(('.', revno), parse_spec('.'),
               check_clean=False,
               ignore_zero=True,
               backup_files=not no_backup,
               file_list=file_list)
         if not file_list:
-            Branch('.').set_pending_merges([])
+            Branch.open_containing('.').set_pending_merges([])
 
 
 class cmd_assert_fail(Command):
@@ -1283,7 +1321,7 @@
         if verbose and quiet:
             raise BzrCommandError('Cannot pass both quiet and verbose')
 
-        b = find_branch('.')
+        b = Branch.open_containing('.')
         parent = b.get_parent()
         if remote is None:
             if parent is None:
@@ -1293,9 +1331,10 @@
                     print "Using last location: %s" % parent
                 remote = parent
         elif parent is None:
-            # We only update parent if it did not exist, missing should not change the parent
+            # We only update parent if it did not exist, missing
+            # should not change the parent
             b.set_parent(remote)
-        br_remote = find_branch(remote)
+        br_remote = Branch.open_containing(remote)
 
         return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
 

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-08-27 04:42:41 +0000
+++ bzrlib/changeset.py	2005-09-26 08:56:15 +0000
@@ -18,6 +18,7 @@
 import patch
 import stat
 from bzrlib.trace import mutter
+from bzrlib.osutils import rename
 
 # XXX: mbp: I'm not totally convinced that we should handle conflicts
 # as part of changeset application, rather than only in the merge
@@ -42,7 +43,6 @@
         newdict[value] = key
     return newdict
 
-
        
 class ChangeUnixPermissions(object):
     """This is two-way change, suitable for file modification, creation,
@@ -92,6 +92,7 @@
     def __ne__(self, other):
         return not (self == other)
 
+
 def dir_create(filename, conflict_handler, reverse):
     """Creates the directory, or deletes it if reverse is true.  Intended to be
     used with ReplaceContents.
@@ -117,14 +118,12 @@
         try:
             os.rmdir(filename)
         except OSError, e:
-            if e.errno != 39:
+            if e.errno != errno.ENOTEMPTY:
                 raise
             if conflict_handler.rmdir_non_empty(filename) == "skip":
                 return
             os.rmdir(filename)
 
-                
-            
 
 class SymlinkCreate(object):
     """Creates or deletes a symlink (for use with ReplaceContents)"""
@@ -353,7 +352,7 @@
         status = patch.diff3(new_file, filename, base, other)
         if status == 0:
             os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
+            rename(new_file, filename)
             return
         else:
             assert(status == 1)
@@ -831,7 +830,7 @@
             if src_path is not None:
                 src_path = os.path.join(dir, src_path)
                 try:
-                    os.rename(src_path, to_name)
+                    rename(src_path, to_name)
                     temp_name[entry.id] = to_name
                 except OSError, e:
                     if e.errno != errno.ENOENT:
@@ -874,7 +873,7 @@
             if old_path is None:
                 continue
             try:
-                os.rename(old_path, new_path)
+                rename(old_path, new_path)
                 changed_inventory[entry.id] = new_tree_path
             except OSError, e:
                 raise Exception ("%s is missing" % new_path)
@@ -1003,9 +1002,6 @@
     descend from this class if they have a better way to handle some or
     all types of conflict.
     """
-    def __init__(self, dir):
-        self.dir = dir
-    
     def missing_parent(self, pathname):
         parent = os.path.dirname(pathname)
         raise Exception("Parent directory missing for %s" % pathname)
@@ -1084,7 +1080,7 @@
     :rtype: Dictionary
     """
     if conflict_handler is None:
-        conflict_handler = ExceptionConflictHandler(dir)
+        conflict_handler = ExceptionConflictHandler()
     temp_dir = os.path.join(dir, "bzr-tree-change")
     try:
         os.mkdir(temp_dir)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-09-15 21:35:53 +0000
+++ bzrlib/commands.py	2005-09-18 00:57:26 +0000
@@ -42,7 +42,7 @@
 import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
 from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
-from bzrlib.branch import find_branch
+from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
 
 plugin_cmds = {}
@@ -76,70 +76,68 @@
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
 
-    This always returns a list.  The list will have one element for 
-
-    It supports integers directly, but everything else it
-    defers for passing to Branch.get_revision_info()
+    This always returns a list.  The list will have one element for
+    each revision.
 
     >>> _parse_revision_str('234')
-    [234]
+    [<RevisionSpec_int 234>]
     >>> _parse_revision_str('234..567')
-    [234, 567]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
     >>> _parse_revision_str('..')
-    [None, None]
+    [<RevisionSpec None>, <RevisionSpec None>]
     >>> _parse_revision_str('..234')
-    [None, 234]
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
     >>> _parse_revision_str('234..')
-    [234, None]
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
     >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [234, 456, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('234....789') # Error?
-    [234, None, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('revid:test@other.com-234234')
-    ['revid:test@other.com-234234']
+    [<RevisionSpec_revid revid:test@other.com-234234>]
     >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    ['revid:test@other.com-234234', 'revid:test@other.com-234235']
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
     >>> _parse_revision_str('revid:test@other.com-234234..23')
-    ['revid:test@other.com-234234', 23]
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('date:2005-04-12')
-    ['date:2005-04-12']
+    [<RevisionSpec_date date:2005-04-12>]
     >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    ['date:2005-04-12 12:24:33']
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
     >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    ['date:2005-04-12T12:24:33']
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
     >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    ['date:2005-04-12,12:24:33']
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
     >>> _parse_revision_str('-5..23')
-    [-5, 23]
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('-5')
-    [-5]
+    [<RevisionSpec_int -5>]
     >>> _parse_revision_str('123a')
-    ['123a']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
     >>> _parse_revision_str('abc')
-    ['abc']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
     """
     import re
     old_format_re = re.compile('\d*:\d*')
     m = old_format_re.match(revstr)
+    revs = []
     if m:
         warning('Colon separator for revision numbers is deprecated.'
                 ' Use .. instead')
-        revs = []
         for rev in revstr.split(':'):
             if rev:
-                revs.append(int(rev))
-            else:
-                revs.append(None)
-        return revs
-    revs = []
-    for x in revstr.split('..'):
-        if not x:
-            revs.append(None)
-        else:
-            try:
-                revs.append(int(x))
-            except ValueError:
-                revs.append(x)
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            else:
+                revs.append(RevisionSpec(x))
     return revs
 
 
@@ -408,13 +406,13 @@
     >>> parse_args('commit --message=biter'.split())
     (['commit'], {'message': u'biter'})
     >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [500]})
+    (['log'], {'revision': [<RevisionSpec_int 500>]})
     >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [500, 600]})
+    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
     >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [500, 600]})
-    >>> parse_args('log -rv500..600'.split()) #the r takes an argument
-    (['log'], {'revision': ['v500', 600]})
+    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
     args = []
     opts = {}
@@ -587,8 +585,6 @@
     --profile
         Run under the Python profiler.
     """
-    # Load all of the transport methods
-    import bzrlib.transport.local, bzrlib.transport.http
     
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
@@ -642,10 +638,14 @@
     bzrlib.trace.log_startup(argv)
     bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
 
+    return run_bzr_catch_errors(argv[1:])
+
+
+def run_bzr_catch_errors(argv):
     try:
         try:
             try:
-                return run_bzr(argv[1:])
+                return run_bzr(argv)
             finally:
                 # do this here inside the exception wrappers to catch EPIPE
                 sys.stdout.flush()
@@ -676,5 +676,6 @@
             bzrlib.trace.log_exception()
             return 2
 
+
 if __name__ == '__main__':
     sys.exit(main(sys.argv))

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-09-15 21:35:53 +0000
+++ bzrlib/commit.py	2005-09-16 03:32:44 +0000
@@ -57,7 +57,7 @@
         If null (default), a time/random revision id is generated.
     """
 
-    import time, tempfile
+    import time, tempfile, re
 
     from bzrlib.osutils import local_time_offset, username
     from bzrlib.branch import gen_file_id
@@ -144,6 +144,22 @@
             timezone = local_time_offset()
 
         mutter("building commit log message")
+        # Python strings can include characters that can't be
+        # represented in well-formed XML; escape characters that
+        # aren't listed in the XML specification
+        # (http://www.w3.org/TR/REC-xml/#NT-Char).
+        if isinstance(message, unicode):
+            char_pattern = u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]'
+        else:
+            # Use a regular 'str' as pattern to avoid having re.subn
+            # return 'unicode' results.
+            char_pattern = '[^x09\x0A\x0D\x20-\xFF]'
+        message, escape_count = re.subn(
+            char_pattern,
+            lambda match: match.group(0).encode('unicode_escape'),
+            message)
+        if escape_count:
+            note("replaced %d control characters in message", escape_count)
         rev = Revision(timestamp=timestamp,
                        timezone=timezone,
                        committer=committer,

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-09-15 13:11:01 +0000
+++ bzrlib/diff.py	2005-09-16 12:53:47 +0000
@@ -161,15 +161,15 @@
         import sys
         output = sys.stdout
 
-    if revision == None:
+    if revision is None:
         old_tree = b.basis_tree()
     else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
+        old_tree = b.revision_tree(revision.in_history(b).rev_id)
 
-    if revision2 == None:
+    if revision2 is None:
         new_tree = b.working_tree()
     else:
-        new_tree = b.revision_tree(b.lookup_revision(revision2))
+        new_tree = b.revision_tree(revision2.in_history(b).rev_id)
 
     show_diff_trees(old_tree, new_tree, output, specific_files,
                     external_diff_options)

=== modified file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	2005-09-01 11:53:02 +0000
+++ bzrlib/externalcommand.py	2005-09-26 08:56:15 +0000
@@ -21,6 +21,7 @@
 
 import os
 import sys
+
 from bzrlib.commands import Command
 
 
@@ -43,7 +44,7 @@
     def __init__(self, path):
         self.path = path
 
- 
+
     def name(self):
         return self.path.split(os.sep)[-1]
 

=== modified file 'bzrlib/graph.py'
--- bzrlib/graph.py	2005-09-12 02:53:07 +0000
+++ bzrlib/graph.py	2005-09-21 15:33:23 +0000
@@ -30,7 +30,7 @@
             continue
         if ancestor not in distances:
             return None
-        if best is None or distances[ancestor] > best:
+        if best is None or distances[ancestor]+1 > best:
             best = distances[ancestor] + 1
     return best
 
@@ -67,16 +67,27 @@
         lines = new_lines
     return distances
 
-def farthest_nodes(graph, ancestors, start):
+def nodes_by_distance(distances):
+    """Return a list of nodes sorted by distance"""
     def by_distance(n):
         return distances[n],n
 
-    distances = node_distances(graph, ancestors, start)
     node_list = distances.keys()
     node_list.sort(key=by_distance, reverse=True)
     return node_list
 
+def select_farthest(distances, common):
+    """Return the farthest common node, or None if no node qualifies."""
+    node_list = nodes_by_distance(distances)
+    for node in node_list:
+        if node in common:
+            return node
+
 def all_descendants(descendants, start):
+    """Produce a set of all descendants of the start node.
+    The input is a map of node->list of descendants for a graph encompassing
+    start.
+    """
     result = set()
     lines = set([start])
     while len(lines) > 0:

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-06 11:09:03 +0000
+++ bzrlib/help.py	2005-09-16 03:32:44 +0000
@@ -30,24 +30,24 @@
 
 Basic commands:
 
-  bzr init      makes this branch versioned
-  bzr branch    make a copy of another branch
-
-  bzr add       make files or directories versioned
-  bzr ignore    ignore a file or pattern
-  bzr mv        move or rename a versioned file
-
-  bzr status    summarize changes in working copy
-  bzr diff      show detailed diffs
-
-  bzr merge     pull in changes from another branch
-  bzr commit    save some or all changes
-
-  bzr log       show history of changes
-  bzr check     validate storage
-
-Use e.g. 'bzr help init' for more details, or
-'bzr help commands' for all commands.
+  bzr init           makes this directory a versioned branch
+  bzr branch         make a copy of another branch
+
+  bzr add            make files or directories versioned
+  bzr ignore         ignore a file or pattern
+  bzr mv             move or rename a versioned file
+
+  bzr status         summarize changes in working copy
+  bzr diff           show detailed diffs
+
+  bzr merge          pull in changes from another branch
+  bzr commit         save some or all changes
+
+  bzr log            show history of changes
+  bzr check          validate storage
+
+  bzr help init      more help on e.g. init command
+  bzr help commands  list all commands
 """
 
 

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2005-05-19 08:31:06 +0000
+++ bzrlib/info.py	2005-09-26 08:56:15 +0000
@@ -18,7 +18,7 @@
 
 import time
 
-from osutils import format_date
+from bzrlib.osutils import format_date
 
 
 def _countiter(it):

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-09-06 07:26:13 +0000
+++ bzrlib/inventory.py	2005-09-26 08:56:15 +0000
@@ -20,15 +20,18 @@
 ROOT_ID = "TREE_ROOT"
 
 
-import sys, os.path, types, re
+import os.path
+import re
+import sys
+import types
 
 import bzrlib
 from bzrlib.errors import BzrError, BzrCheckError
 
-from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
+from bzrlib.osutils import quotefn, splitpath, joinpath, appendpath
 from bzrlib.trace import mutter
 from bzrlib.errors import NotVersionedError
-        
+
 
 class InventoryEntry(object):
     """Description of a versioned file.
@@ -79,16 +82,16 @@
     InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
     >>> i['2326']
     InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
-    >>> for j in i.iter_entries():
-    ...     print j[0]
-    ...     assert i.path2id(j[0])
+    >>> for path, entry in i.iter_entries():
+    ...     print path.replace('\\\\', '/')     # for win32 os.sep
+    ...     assert i.path2id(path)
     ... 
     src
     src/bye.c
     src/hello.c
     src/wibble
     src/wibble/wibble.c
-    >>> i.id2path('2326')
+    >>> i.id2path('2326').replace('\\\\', '/')
     'src/wibble/wibble.c'
 
     TODO: Maybe also keep the full path of the entry, and the children?
@@ -254,7 +257,7 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch(init=True) create a unique inventory
+        # We are letting Branch.initialize() create a unique inventory
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2005-07-21 12:01:14 +0000
+++ bzrlib/lock.py	2005-09-19 07:18:30 +0000
@@ -168,6 +168,8 @@
                 LOCK_NB = 4
                 def unlock(self):
                     _msvc_unlock(self.f)
+                    self.f.close()
+                    self.f = None
 
 
             class _msvc_ReadLock(_msvc_FileLock):

=== modified file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-08-25 05:58:05 +0000
+++ bzrlib/mdiff.py	2005-09-26 08:56:15 +0000
@@ -27,10 +27,13 @@
 
 # FIXME: doesn't work properly on files without trailing newlines
 
+import difflib
+import struct
+import sys
 import unittest
-import difflib, sys, struct
 from cStringIO import StringIO
 
+
 def linesplit(a):
     """Split into two lists: content and line positions.
 

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-09-14 16:35:46 +0000
+++ bzrlib/merge.py	2005-09-26 08:56:15 +0000
@@ -26,13 +26,15 @@
 from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
 from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
 from bzrlib.changeset import Inventory, Diff3Merge
-from bzrlib.branch import find_branch
+from bzrlib.branch import Branch
 from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
 from bzrlib.errors import NoCommits
 from bzrlib.delta import compare_trees
 from bzrlib.trace import mutter, warning
 from bzrlib.fetch import greedy_fetch
 from bzrlib.revision import is_ancestor
+from bzrlib.osutils import rename
+
 
 # comments from abentley on irc: merge happens in two stages, each
 # of which generates a changeset object
@@ -47,8 +49,8 @@
     conflict that are not explicitly handled cause an exception and
     terminate the merge.
     """
-    def __init__(self, dir, ignore_zero=False):
-        ExceptionConflictHandler.__init__(self, dir)
+    def __init__(self, ignore_zero=False):
+        ExceptionConflictHandler.__init__(self)
         self.conflicts = 0
         self.ignore_zero = ignore_zero
 
@@ -84,7 +86,7 @@
             last_new_name = name
         new_name = last_new_name+suffix
         try:
-            os.rename(name, new_name)
+            rename(name, new_name)
             return new_name
         except OSError, e:
             if e.errno != errno.EEXIST and e.errno != errno.ENOTEMPTY:
@@ -108,7 +110,7 @@
         self.add_suffix(this_path, ".THIS")
         self.dump(base_lines, this_path+".BASE")
         self.dump(other_lines, this_path+".OTHER")
-        os.rename(new_file, this_path)
+        rename(new_file, this_path)
         self.conflict("Diff3 conflict encountered in %s" % this_path)
 
     def new_contents_conflict(self, filename, other_contents):
@@ -134,13 +136,13 @@
             
 def get_tree(treespec, temp_root, label, local_branch=None):
     location, revno = treespec
-    branch = find_branch(location)
+    branch = Branch.open_containing(location)
     if revno is None:
         revision = None
     elif revno == -1:
         revision = branch.last_patch()
     else:
-        revision = branch.lookup_revision(revno)
+        revision = branch.get_rev_id(revno)
     return branch, get_revid_tree(branch, revision, temp_root, label,
                                   local_branch)
 
@@ -244,7 +246,7 @@
     try:
         if this_dir is None:
             this_dir = '.'
-        this_branch = find_branch(this_dir)
+        this_branch = Branch.open_containing(this_dir)
         this_rev_id = this_branch.last_patch()
         if this_rev_id is None:
             raise BzrCommandError("This branch has no commits")
@@ -261,7 +263,7 @@
                 raise NoCommits(other_branch)
             other_basis = other_rev_id
         elif other_revision[1] is not None:
-            other_rev_id = other_branch.lookup_revision(other_revision[1])
+            other_rev_id = other_branch.get_rev_id(other_revision[1])
             other_basis = other_rev_id
         else:
             other_rev_id = None
@@ -284,13 +286,10 @@
             elif base_revision[1] is None:
                 base_rev_id = None
             else:
-                base_rev_id = base_branch.lookup_revision(base_revision[1])
-            if base_rev_id is not None:
-                base_is_ancestor = is_ancestor(this_rev_id, base_rev_id, 
-                                               MultipleRevisionSources(this_branch, 
-                                                                       base_branch))
-            else:
-                base_is_ancestor = False
+                base_rev_id = base_branch.get_rev_id(base_revision[1])
+            multi_source = MultipleRevisionSources(this_branch, base_branch)
+            base_is_ancestor = is_ancestor(this_rev_id, base_rev_id,
+                                           multi_source)
         if file_list is None:
             interesting_ids = None
         else:
@@ -352,8 +351,7 @@
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(base_tree.root,
-                                                  ignore_zero=ignore_zero),
+                             MergeConflictHandler(ignore_zero=ignore_zero),
                              merge_factory=merge_factory, 
                              interesting_ids=interesting_ids)
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-08-27 04:42:41 +0000
+++ bzrlib/merge_core.py	2005-09-23 09:25:16 +0000
@@ -1,7 +1,7 @@
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
 import os.path
-from osutils import backup_file
+from osutils import backup_file, rename
 from merge3 import Merge3
 
 class ApplyMerge3:
@@ -51,7 +51,7 @@
         output_file.close()
         if not new_conflicts:
             os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
+            rename(new_file, filename)
             return
         else:
             conflict_handler.merge_conflict(new_file, filename, base_lines,

=== modified file 'bzrlib/meta_store.py'
--- bzrlib/meta_store.py	2005-09-15 21:35:53 +0000
+++ bzrlib/meta_store.py	2005-09-26 08:56:15 +0000
@@ -14,23 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter
-from bzrlib.store import Storage, CompressedTextStore
-from bzrlib.local_transport import LocalTransport
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-class CachedStore(Storage):
+from bzrlib.trace import mutter
+from bzrlib.store import ImmutableStore
+
+
+class CachedStore:
     """A store that caches data locally, to avoid repeated downloads.
     The precacache method should be used to avoid server round-trips for
     every piece of data.
     """
     def __init__(self, store, cache_dir):
         self.source_store = store
-        self.cache_store = CompressedTextStore(LocalTransport(cache_dir))
+        self.cache_store = ImmutableStore(cache_dir)
 
     def __getitem__(self, id):
         mutter("Cache add %s" % id)
@@ -38,25 +33,6 @@
             self.cache_store.add(self.source_store[id], id)
         return self.cache_store[id]
 
-    def __contains__(self, fileid):
-        if fileid in self.cache_store:
-            return True
-        if fileid in self.source_store:
-            # We could copy at this time
-            return True
-        return False
-
-    def get(self, fileids, pb=None):
-        fileids = list(fileids)
-        hasids = self.cache_store.has(fileids)
-        needs = set()
-        for has, fileid in zip(hasids, fileids):
-            if not has:
-                needs.add(fileid)
-        if needs:
-            self.cache_store.copy_multi(self.source_store, needs)
-        return self.cache_store.get(fileids, pb=pb)
-
     def prefetch(self, ids):
         """Copy a series of ids into the cache, before they are used.
         For remote stores that support pipelining or async downloads, this can

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-09-07 01:34:07 +0000
+++ bzrlib/msgeditor.py	2005-09-19 06:16:22 +0000
@@ -27,6 +27,11 @@
     """Return a sequence of possible editor binaries for the current platform"""
     from bzrlib.osutils import _read_config_value
     
+    try:
+        yield os.environ["BZR_EDITOR"]
+    except KeyError:
+        pass
+
     e = _read_config_value("editor")
     if e is not None:
         yield e
@@ -34,10 +39,12 @@
     try:
         yield os.environ["EDITOR"]
     except KeyError:
-        if os.name == "nt":
-            yield "notepad.exe"
-        elif os.name == "posix":
-            yield "/usr/bin/vi"
+        pass
+
+    if os.name == "nt":
+        yield "notepad.exe"
+    elif os.name == "posix":
+        yield "/usr/bin/vi"
 
 
 def _run_editor(filename):

=== modified file 'bzrlib/newinventory.py'
--- bzrlib/newinventory.py	2005-07-11 07:33:45 +0000
+++ bzrlib/newinventory.py	2005-09-26 08:56:15 +0000
@@ -14,13 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from xml import ElementTree, Element
+from bzrlib.xml import ElementTree, Element
 
 
 def write_inventory(inv, f):
     el = Element('inventory', {'version': '2'})
     el.text = '\n'
-    
+
     root = Element('root_directory', {'id': inv.root.file_id})
     root.tail = root.text = '\n'
     el.append(root)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-09-09 14:17:50 +0000
+++ bzrlib/osutils.py	2005-09-26 08:56:15 +0000
@@ -1,31 +1,38 @@
 # Bazaar-NG -- distributed version control
-
+#
 # Copyright (C) 2005 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import os, types, re, time, errno, sys
-from stat import S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE
+from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
+                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+import errno
+import os
+import re
+import sha
+import sys
+import time
+import types
 
+import bzrlib
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
-import bzrlib
+
 
 def make_readonly(filename):
     """Make a filename read-only."""
-    # TODO: probably needs to be fixed for windows
     mod = os.stat(filename).st_mode
     mod = mod & 0777555
     os.chmod(filename, mod)
@@ -48,7 +55,7 @@
     # TODO: I'm not really sure this is the best format either.x
     global _QUOTE_RE
     if _QUOTE_RE == None:
-        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/\\_~-])')
         
     if _QUOTE_RE.search(f):
         return '"' + f + '"'
@@ -64,8 +71,16 @@
         return 'directory'
     elif S_ISLNK(mode):
         return 'symlink'
+    elif S_ISCHR(mode):
+        return 'chardev'
+    elif S_ISBLK(mode):
+        return 'block'
+    elif S_ISFIFO(mode):
+        return 'fifo'
+    elif S_ISSOCK(mode):
+        return 'socket'
     else:
-        raise BzrError("can't handle file kind with mode %o of %r" % (mode, f))
+        return 'unknown'
 
 
 def kind_marker(kind):
@@ -79,7 +94,6 @@
         raise BzrError('invalid file kind %r' % kind)
 
 
-
 def backup_file(fn):
     """Copy a file to a backup.
 
@@ -87,7 +101,6 @@
 
     If the file is already a backup, it's not copied.
     """
-    import os
     if fn[-1] == '~':
         return
     bfn = fn + '~'
@@ -104,18 +117,11 @@
     finally:
         outf.close()
 
-def rename(path_from, path_to):
-    """Basically the same as os.rename() just special for win32"""
-    if sys.platform == 'win32':
-        try:
-            os.remove(path_to)
-        except OSError, e:
-            if e.errno != e.ENOENT:
-                raise
-    os.rename(path_from, path_to)
-
-
-
+if os.name == 'nt':
+    import shutil
+    rename = shutil.move
+else:
+    rename = os.rename
 
 
 def isdir(f):
@@ -126,7 +132,6 @@
         return False
 
 
-
 def isfile(f):
     """True if f is a regular file."""
     try:
@@ -145,11 +150,11 @@
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
     
-    >>> is_inside('src', 'src/foo.c')
+    >>> is_inside('src', os.path.join('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', 'src/a/a/a/foo.c')
+    >>> is_inside('src', os.path.join('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
@@ -165,10 +170,10 @@
     
     if dir == '':
         return True
-    
+
     if dir[-1] != os.sep:
         dir += os.sep
-    
+
     return fname.startswith(dir)
 
 
@@ -186,16 +191,7 @@
     tofile.write(fromfile.read())
 
 
-def uuid():
-    """Return a new UUID"""
-    try:
-        return file('/proc/sys/kernel/random/uuid').readline().rstrip('\n')
-    except IOError:
-        return chomp(os.popen('uuidgen').readline())
-
-
 def sha_file(f):
-    import sha
     if hasattr(f, 'tell'):
         assert f.tell() == 0
     s = sha.new()
@@ -209,15 +205,12 @@
 
 
 def sha_string(f):
-    import sha
     s = sha.new()
     s.update(f)
     return s.hexdigest()
 
 
-
 def fingerprint_file(f):
-    import sha
     s = sha.new()
     b = f.read()
     s.update(b)
@@ -233,7 +226,7 @@
     
     TODO: Global option --config-dir to override this.
     """
-    return os.path.expanduser("~/.bzr.conf")
+    return os.path.join(os.path.expanduser("~"), ".bzr.conf")
 
 
 def _auto_user_id():
@@ -343,11 +336,11 @@
     if e:
         m = re.search(r'[\w+.-]+@[\w+.-]+', e)
         if not m:
-            raise BzrError("%r doesn't seem to contain a reasonable email address" % e)
+            raise BzrError("%r doesn't seem to contain "
+                           "a reasonable email address" % e)
         return m.group(0)
 
     return _auto_user_id()[1]
-    
 
 
 def compare_files(a, b):
@@ -362,7 +355,6 @@
             return True
 
 
-
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
     # python2.3 localtime() can't take None
@@ -496,5 +488,3 @@
         if e.errno == errno.ENOENT:
             return None
         raise
-
-

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2005-08-25 05:58:05 +0000
+++ bzrlib/patch.py	2005-09-23 09:25:16 +0000
@@ -6,8 +6,13 @@
 __docformat__ = "restructuredtext"
 
 def write_to_cmd(args, input=""):
-    process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
-                    stderr=PIPE, close_fds=True)
+    if os.name != 'nt':
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE, close_fds=True)
+    else:
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE)
+
     stdout, stderr = process.communicate(input)
     status = process.wait()
     if status < 0:

=== modified file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-09-15 13:32:36 +0000
+++ bzrlib/remotebranch.py	2005-09-21 06:07:21 +0000
@@ -29,7 +29,7 @@
 import urlparse
 
 from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
+from bzrlib.branch import Branch, LocalBranch, BZR_BRANCH_FORMAT
 from bzrlib.trace import mutter
 from bzrlib.xml import serializer_v4
 
@@ -110,7 +110,7 @@
         
 
 
-class RemoteBranch(Branch):
+class RemoteBranch(LocalBranch):
     def __init__(self, baseurl, find_root=True):
         """Create new proxy for a remote branch."""
         if find_root:
@@ -129,6 +129,16 @@
 
     __repr__ = __str__
 
+    def setup_caching(self, cache_root):
+        """Set up cached stores located under cache_root"""
+        from bzrlib.meta_store import CachedStore
+        for store_name in ('inventory_store', 'text_store', 'revision_store'):
+            if not isinstance(getattr(self, store_name), CachedStore):
+                cache_path = os.path.join(cache_root, store_name)
+                os.mkdir(cache_path)
+                new_store = CachedStore(getattr(self, store_name), cache_path)
+                setattr(self, store_name, new_store)
+
     def controlfile(self, filename, mode):
         if mode not in ('rb', 'rt', 'r'):
             raise BzrError("file mode %r not supported for remote branches" % mode)
@@ -183,6 +193,10 @@
         try:
             return get_url(p, compressed=True)
         except urllib2.URLError:
+            pass
+        try:
+            return get_url(p, compressed=False)
+        except urllib2.URLError:
             raise KeyError(fileid)
     
 

=== modified file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-08-25 05:58:05 +0000
+++ bzrlib/revfile.py	2005-09-26 08:56:15 +0000
@@ -93,9 +93,17 @@
 # then ftell after writing to see where it went.  In any case we
 # assume the whole branch is protected by a lock.
 
-import sys, zlib, struct, mdiff, stat, os, sha
+import os
+import sha
+import stat
+import struct
+import sys
+import zlib
 from binascii import hexlify, unhexlify
 
+import bzrlib.mdiff as mdiff
+
+
 _RECORDSIZE = 48
 
 _HEADER = "bzr revfile v1\n"

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-09-14 17:23:14 +0000
+++ bzrlib/revision.py	2005-09-20 14:30:35 +0000
@@ -16,7 +16,9 @@
 
 
 import bzrlib.errors
-from bzrlib.graph import farthest_nodes, node_distances, all_descendants
+from bzrlib.graph import node_distances, select_farthest, all_descendants
+
+NULL_REVISION="null:"
 
 class RevisionReference(object):
     """
@@ -106,7 +108,8 @@
     revisions_source is an object supporting a get_revision operation that
     behaves like Branch's.
     """
-
+    if candidate_id is None:
+        return True
     for ancestor_id, distance in iter_ancestors(revision_id, revision_source):
         if ancestor_id == candidate_id:
             return True
@@ -206,15 +209,19 @@
     while len(lines) > 0:
         new_lines = set()
         for line in lines:
-            try:
-                rev = revision_source.get_revision(line)
-                parents = [p.revision_id for p in rev.parents]
-                if len(parents) == 0:
-                    root = line
-            except bzrlib.errors.NoSuchRevision:
-                if line == revision:
-                    raise
-                parents = None
+            if line == NULL_REVISION:
+                parents = []
+                root = NULL_REVISION
+            else:
+                try:
+                    rev = revision_source.get_revision(line)
+                    parents = [p.revision_id for p in rev.parents]
+                    if len(parents) == 0:
+                        parents = [NULL_REVISION]
+                except bzrlib.errors.NoSuchRevision:
+                    if line == revision:
+                        raise
+                    parents = None
             if parents is not None:
                 for parent in parents:
                     if parent not in ancestors:
@@ -246,7 +253,7 @@
         ancestors[node].update(node_anc)
     for node, node_dec in descendants_b.iteritems():
         if node not in descendants:
-            descendants[node] = set()
+            descendants[node] = {}
         descendants[node].update(node_dec)
     return root, ancestors, descendants, common
 
@@ -257,11 +264,11 @@
     except bzrlib.errors.NoCommonRoot:
         raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
         
-    nodes = farthest_nodes(descendants, ancestors, root)
-    for node in nodes:
-        if node in common:
-            return node
-    raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    distances = node_distances (descendants, ancestors, root)
+    farthest = select_farthest(distances, common)
+    if farthest is None or farthest == NULL_REVISION:
+        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    return farthest
 
 class MultipleRevisionSources(object):
     """Proxy that looks in multiple branches for revisions."""

=== added file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revisionspec.py	2005-09-23 08:54:16 +0000
@@ -0,0 +1,320 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import datetime
+import re
+from bzrlib.errors import BzrError, NoSuchRevision, NoCommits
+
+_marker = []
+
+class RevisionInfo(object):
+    """The results of applying a revision specification to a branch.
+
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    This class exists mostly to be the return value of a RevisionSpec,
+    so that you can access the member you're interested in (number or id)
+    or treat the result as a tuple.
+    """
+
+    def __init__(self, branch, revno, rev_id=_marker):
+        self.branch = branch
+        self.revno = revno
+        if rev_id is _marker:
+            # allow caller to be lazy
+            if self.revno is None:
+                self.rev_id = None
+            else:
+                self.rev_id = branch.get_rev_id(self.revno)
+        else:
+            self.rev_id = rev_id
+
+    def __nonzero__(self):
+        # first the easy ones...
+        if self.rev_id is None:
+            return False
+        if self.revno is not None:
+            return True
+        # TODO: otherwise, it should depend on how I was built -
+        # if it's in_history(branch), then check revision_history(),
+        # if it's in_store(branch), do the check below
+        return self.rev_id in self.branch.revision_store
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
+
+# classes in this list should have a "prefix" attribute, against which
+# string specs are matched
+SPEC_TYPES = []
+
+class RevisionSpec(object):
+    """A parsed revision specification.
+
+    A revision specification can be an integer, in which case it is
+    assumed to be a revno (though this will translate negative values
+    into positive ones); or it can be a string, in which case it is
+    parsed for something like 'date:' or 'revid:' etc.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
+
+    (Equivalent to the old Branch method get_revision_info())
+    """
+
+    prefix = None
+
+    def __new__(cls, spec, foo=_marker):
+        """Parse a revision specifier.
+        """
+        if spec is None:
+            return object.__new__(RevisionSpec, spec)
+
+        try:
+            spec = int(spec)
+        except ValueError:
+            pass
+
+        if isinstance(spec, int):
+            return object.__new__(RevisionSpec_int, spec)
+        elif isinstance(spec, basestring):
+            for spectype in SPEC_TYPES:
+                if spec.startswith(spectype.prefix):
+                    return object.__new__(spectype, spec)
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+    def __init__(self, spec):
+        if self.prefix and spec.startswith(self.prefix):
+            spec = spec[len(self.prefix):]
+        self.spec = spec
+
+    def _match_on(self, branch, revs):
+        return RevisionInfo(branch, 0, None)
+
+    def _match_on_and_check(self, branch, revs):
+        info = self._match_on(branch, revs)
+        if info:
+            return info
+        elif info == (0, None):
+            # special case - the empty tree
+            return info
+        elif self.prefix:
+            raise NoSuchRevision(branch, self.prefix + str(self.spec))
+        else:
+            raise NoSuchRevision(branch, str(self.spec))
+
+    def in_history(self, branch):
+        revs = branch.revision_history()
+        return self._match_on_and_check(branch, revs)
+
+    def __repr__(self):
+        # this is mostly for helping with testing
+        return '<%s %s%s>' % (self.__class__.__name__,
+                              self.prefix or '',
+                              self.spec)
+
+
+# private API
+
+class RevisionSpec_int(RevisionSpec):
+    """Spec is a number.  Special case."""
+    def __init__(self, spec):
+        self.spec = int(spec)
+
+    def _match_on(self, branch, revs):
+        if self.spec < 0:
+            revno = len(revs) + self.spec + 1
+        else:
+            revno = self.spec
+        rev_id = branch.get_rev_id(revno, revs)
+        return RevisionInfo(branch, revno, rev_id)
+
+
+class RevisionSpec_revno(RevisionSpec):
+    prefix = 'revno:'
+
+    def _match_on(self, branch, revs):
+        """Lookup a revision by revision number"""
+        try:
+            return RevisionInfo(branch, int(self.spec))
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revno)
+
+
+class RevisionSpec_revid(RevisionSpec):
+    prefix = 'revid:'
+
+    def _match_on(self, branch, revs):
+        try:
+            return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revid)
+
+
+class RevisionSpec_last(RevisionSpec):
+
+    prefix = 'last:'
+
+    def _match_on(self, branch, revs):
+        try:
+            offset = int(self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+        else:
+            if offset <= 0:
+                raise BzrError('You must supply a positive value for --revision last:XXX')
+            return RevisionInfo(branch, len(revs) - offset + 1)
+
+SPEC_TYPES.append(RevisionSpec_last)
+
+
+class RevisionSpec_before(RevisionSpec):
+
+    prefix = 'before:'
+    
+    def _match_on(self, branch, revs):
+        r = RevisionSpec(self.spec)._match_on(branch, revs)
+        if (r.revno is None) or (r.revno == 0):
+            return r
+        return RevisionInfo(branch, r.revno - 1)
+
+SPEC_TYPES.append(RevisionSpec_before)
+
+
+class RevisionSpec_tag(RevisionSpec):
+    prefix = 'tag:'
+
+    def _match_on(self, branch, revs):
+        raise BzrError('tag: namespace registered, but not implemented.')
+
+SPEC_TYPES.append(RevisionSpec_tag)
+
+
+class RevisionSpec_date(RevisionSpec):
+    prefix = 'date:'
+    _date_re = re.compile(
+            r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+            r'(,|T)?\s*'
+            r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+        )
+
+    def _match_on(self, branch, revs):
+        """
+        Spec for date revisions:
+          date:value
+          value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+          matches the first entry after a given date (either at midnight or
+          at a specified time).
+
+          So the proper way of saying 'give me all entries for today' is:
+              -r date:today..date:tomorrow
+        """
+        today = datetime.datetime.fromordinal(datetime.date.today().toordinal())
+        if self.spec.lower() == 'yesterday':
+            dt = today - datetime.timedelta(days=1)
+        elif self.spec.lower() == 'today':
+            dt = today
+        elif self.spec.lower() == 'tomorrow':
+            dt = today + datetime.timedelta(days=1)
+        else:
+            m = self._date_re.match(self.spec)
+            if not m or (not m.group('date') and not m.group('time')):
+                raise BzrError('Invalid revision date %r' % self.spec)
+
+            if m.group('date'):
+                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
+            else:
+                year, month, day = today.year, today.month, today.day
+            if m.group('time'):
+                hour = int(m.group('hour'))
+                minute = int(m.group('minute'))
+                if m.group('second'):
+                    second = int(m.group('second'))
+                else:
+                    second = 0
+            else:
+                hour, minute, second = 0,0,0
+
+            dt = datetime.datetime(year=year, month=month, day=day,
+                    hour=hour, minute=minute, second=second)
+        first = dt
+        for i in range(len(revs)):
+            r = branch.get_revision(revs[i])
+            # TODO: Handle timezone.
+            dt = datetime.datetime.fromtimestamp(r.timestamp)
+            if first <= dt:
+                return RevisionInfo(branch, i+1)
+        return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_date)
+
+
+class RevisionSpec_ancestor(RevisionSpec):
+    prefix = 'ancestor:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from revision import common_ancestor, MultipleRevisionSources
+        other_branch = Branch.open_containing(self.spec)
+        revision_a = branch.last_patch()
+        revision_b = other_branch.last_patch()
+        for r, b in ((revision_a, branch), (revision_b, other_branch)):
+            if r is None:
+                raise NoCommits(b)
+        revision_source = MultipleRevisionSources(branch, other_branch)
+        rev_id = common_ancestor(revision_a, revision_b, revision_source)
+        try:
+            revno = branch.revision_id_to_revno(rev_id)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, rev_id)
+        
+SPEC_TYPES.append(RevisionSpec_ancestor)

=== modified file 'bzrlib/selftest/HTTPTestUtil.py'
--- bzrlib/selftest/HTTPTestUtil.py	2005-09-14 18:07:16 +0000
+++ bzrlib/selftest/HTTPTestUtil.py	2005-09-23 03:37:57 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import BaseHTTPServer, SimpleHTTPServer
+import BaseHTTPServer, SimpleHTTPServer, socket, errno, time
 from bzrlib.selftest import TestCaseInTempDir
 
 
@@ -27,17 +27,49 @@
 
 class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
     def log_message(self, format, *args):
-        self.server.test_case.log("webserver - %s - - [%s] %s\n" %
+        self.server.test_case.log("webserver - %s - - [%s] %s" %
                                   (self.address_string(),
                                    self.log_date_time_string(),
                                    format%args))
 
+    def handle_one_request(self):
+        """Handle a single HTTP request.
+
+        You normally don't need to override this method; see the class
+        __doc__ string for information on how to handle specific HTTP
+        commands such as GET and POST.
+
+        """
+        for i in xrange(1,11): # Don't try more than 10 times
+            try:
+                self.raw_requestline = self.rfile.readline()
+            except socket.error, e:
+                if e.args[0] == errno.EAGAIN:
+                    self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
+                    time.sleep(0.01)
+                    continue
+                raise
+            else:
+                break
+        if not self.raw_requestline:
+            self.close_connection = 1
+            return
+        if not self.parse_request(): # An error code has been sent, just exit
+            return
+        mname = 'do_' + self.command
+        if not hasattr(self, mname):
+            self.send_error(501, "Unsupported method (%r)" % self.command)
+            return
+        method = getattr(self, mname)
+        method()
+
 class TestingHTTPServer(BaseHTTPServer.HTTPServer):
     def __init__(self, server_address, RequestHandlerClass, test_case):
         BaseHTTPServer.HTTPServer.__init__(self, server_address,
                                                 RequestHandlerClass)
         self.test_case = test_case
 
+
 class TestCaseWithWebserver(TestCaseInTempDir):
     """Derived class that starts a localhost-only webserver
     (in addition to what TestCaseInTempDir does).
@@ -70,7 +102,7 @@
 
         self._http_base_url = 'http://localhost:%s/' % port
         self._http_starting.release()
-        httpd.socket.settimeout(1)
+        httpd.socket.settimeout(0.1)
 
         while self._http_running:
             try:
@@ -95,7 +127,7 @@
         return self._http_base_url + remote_path
 
     def setUp(self):
-        super(TestCaseWithWebserver, self).setUp()
+        TestCaseInTempDir.setUp(self)
         import threading, os
         self._local_path_parts = self.test_dir.split(os.path.sep)
         self._http_starting = threading.Lock()
@@ -109,4 +141,5 @@
     def tearDown(self):
         self._http_running = False
         self._http_thread.join()
-        super(TestCaseWithWebserver, self).tearDown()
+        TestCaseInTempDir.tearDown(self)
+

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/__init__.py	2005-09-26 08:56:15 +0000
@@ -15,16 +15,18 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+from cStringIO import StringIO
 import logging
 import unittest
 import tempfile
 import os
 import sys
+import errno
 import subprocess
+import shutil
+import testsweet
 
-from testsweet import run_suite
 import bzrlib.commands
-
 import bzrlib.trace
 import bzrlib.fetch
 
@@ -88,6 +90,55 @@
         """Return as a string the log for this test"""
         return open(self._log_file_name).read()
 
+
+    def capture(self, cmd):
+        """Shortcut that splits cmd into words, runs, and returns stdout"""
+        return self.run_bzr_captured(cmd.split())[0]
+
+    def run_bzr_captured(self, argv, retcode=0):
+        """Invoke bzr and return (result, stdout, stderr).
+
+        Useful for code that wants to check the contents of the
+        output, the way error messages are presented, etc.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        Much of the old code runs bzr by forking a new copy of Python, but
+        that is slower, harder to debug, and generally not necessary.
+
+        This runs bzr through the interface that catches and reports
+        errors, and with logging set to something approximating the
+        default, so that error reporting can be checked.
+
+        argv -- arguments to invoke bzr
+        retcode -- expected return code, or None for don't-care.
+        """
+        stdout = StringIO()
+        stderr = StringIO()
+        self.log('run bzr: %s', ' '.join(argv))
+        handler = logging.StreamHandler(stderr)
+        handler.setFormatter(bzrlib.trace.QuietFormatter())
+        handler.setLevel(logging.INFO)
+        logger = logging.getLogger('')
+        logger.addHandler(handler)
+        try:
+            result = self.apply_redirected(None, stdout, stderr,
+                                           bzrlib.commands.run_bzr_catch_errors,
+                                           argv)
+        finally:
+            logger.removeHandler(handler)
+        out = stdout.getvalue()
+        err = stderr.getvalue()
+        if out:
+            self.log('output:\n%s', out)
+        if err:
+            self.log('errors:\n%s', err)
+        if retcode is not None:
+            self.assertEquals(result, retcode)
+        return out, err
+
     def run_bzr(self, *args, **kwargs):
         """Invoke bzr, as if it were run from the command line.
 
@@ -95,15 +146,12 @@
         overall behavior of the bzr application (rather than a unit test
         or a functional test of the library.)
 
-        Much of the old code runs bzr by forking a new copy of Python, but
-        that is slower, harder to debug, and generally not necessary.
+        This sends the stdout/stderr results into the test's log,
+        where it may be useful for debugging.  See also run_captured.
         """
-        retcode = kwargs.get('retcode', 0)
-        result = self.apply_redirected(None, None, None,
-                                       bzrlib.commands.run_bzr, args)
-        self.assertEquals(result, retcode)
-        
-        
+        retcode = kwargs.pop('retcode', 0)
+        return self.run_bzr_captured(args, retcode)
+
     def check_inventory_shape(self, inv, shape):
         """
         Compare an inventory to a list of expected names.
@@ -130,7 +178,6 @@
         """Call callable with redirected std io pipes.
 
         Returns the return code."""
-        from StringIO import StringIO
         if not callable(a_callable):
             raise ValueError("a_callable must be callable.")
         if stdin is None:
@@ -185,27 +232,31 @@
         if contents != expect:
             self.log("expected: %r" % expect)
             self.log("actually: %r" % contents)
-            self.fail("contents of %s not as expected")
+            self.fail("contents of %s not as expected" % filename)
 
     def _make_test_root(self):
-        import os
-        import shutil
-        import tempfile
-        
         if TestCaseInTempDir.TEST_ROOT is not None:
             return
-        TestCaseInTempDir.TEST_ROOT = os.path.abspath(
-                                 tempfile.mkdtemp(suffix='.tmp',
-                                                  prefix=self._TEST_NAME + '-',
-                                                  dir=os.curdir))
-    
+        i = 0
+        while True:
+            root = 'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            break
         # make a fake bzr directory there to prevent any tests propagating
         # up onto the source directory's real branch
         os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
 
     def setUp(self):
         super(TestCaseInTempDir, self).setUp()
-        import os
         self._make_test_root()
         self._currentdir = os.getcwdu()
         self.test_dir = os.path.join(self.TEST_ROOT, self.id())
@@ -213,54 +264,9 @@
         os.chdir(self.test_dir)
         
     def tearDown(self):
-        import os
         os.chdir(self._currentdir)
         super(TestCaseInTempDir, self).tearDown()
 
-    def _formcmd(self, cmd):
-        if isinstance(cmd, basestring):
-            cmd = cmd.split()
-        if cmd[0] == 'bzr':
-            cmd[0] = self.BZRPATH
-            if self.OVERRIDE_PYTHON:
-                cmd.insert(0, self.OVERRIDE_PYTHON)
-        self.log('$ %r' % cmd)
-        return cmd
-
-    def runcmd(self, cmd, retcode=0):
-        """Run one command and check the return code.
-
-        Returns a tuple of (stdout,stderr) strings.
-
-        If a single string is based, it is split into words.
-        For commands that are not simple space-separated words, please
-        pass a list instead."""
-        cmd = self._formcmd(cmd)
-        self.log('$ ' + ' '.join(cmd))
-        actual_retcode = subprocess.call(cmd, stdout=self._log_file,
-                                         stderr=self._log_file)
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-    def backtick(self, cmd, retcode=0):
-        """Run a command and return its output"""
-        cmd = self._formcmd(cmd)
-        child = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=self._log_file)
-        outd, errd = child.communicate()
-        self.log(outd)
-        actual_retcode = child.wait()
-
-        outd = outd.replace('\r', '')
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-        return outd
-
-
-
     def build_tree(self, shape):
         """Build a test tree according to a pattern.
 
@@ -270,7 +276,6 @@
         This doesn't add anything to a branch.
         """
         # XXX: It's OK to just create them using forward slashes on windows?
-        import os
         for name in shape:
             assert isinstance(name, basestring)
             if name[-1] == '/':
@@ -279,7 +284,6 @@
                 f = file(name, 'wt')
                 print >>f, "contents of", name
                 f.close()
-                
 
 
 class MetaTestLog(TestCase):
@@ -292,7 +296,7 @@
 
 
 def selftest(verbose=False, pattern=".*"):
-    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
+    return testsweet.run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern)
 
 
 def test_suite():
@@ -300,10 +304,6 @@
     import bzrlib, bzrlib.store, bzrlib.inventory, bzrlib.branch
     import bzrlib.osutils, bzrlib.commands, bzrlib.merge3, bzrlib.plugin
     from doctest import DocTestSuite
-    import os
-    import shutil
-    import time
-    import sys
 
     global MODULES_TO_TEST, MODULES_TO_DOCTEST
 
@@ -324,13 +324,14 @@
                    'bzrlib.selftest.testbranch',
                    'bzrlib.selftest.testremotebranch',
                    'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.test_revision_info',
                    'bzrlib.selftest.test_merge_core',
                    'bzrlib.selftest.test_smart_add',
+                   'bzrlib.selftest.test_bad_files',
                    'bzrlib.selftest.testdiff',
                    'bzrlib.selftest.test_xml',
                    'bzrlib.fetch',
                    'bzrlib.selftest.teststore',
-                   'bzrlib.selftest.testtransport',
                    'bzrlib.selftest.testgraph',
                    ]
 

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-09-14 17:23:14 +0000
+++ bzrlib/selftest/blackbox.py	2005-09-26 08:56:15 +0000
@@ -26,25 +26,24 @@
 it's normally invoked.
 """
 
+from cStringIO import StringIO
+import os
 import sys
-import os
 
 from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
 from bzrlib.branch import Branch
-from bzrlib.commands import run_bzr
 
 
 class ExternalBase(TestCaseInTempDir):
-    def runbzr(self, args, retcode=0,backtick=False):
+
+    def runbzr(self, args, retcode=0, backtick=False):
         if isinstance(args, basestring):
             args = args.split()
 
         if backtick:
-            return self.backtick(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
+            return self.run_bzr_captured(args, retcode=retcode)[0]
         else:
-            return self.runcmd(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
+            return self.run_bzr_captured(args, retcode=retcode)
 
 
 class TestCommands(ExternalBase):
@@ -73,10 +72,22 @@
         f = file('.bzr/email', 'wt')
         f.write('Branch Identity <branch@identi.ty>')
         f.close()
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
         whoami = self.runbzr("whoami",backtick=True)
         whoami_email = self.runbzr("whoami --email",backtick=True)
         self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
         self.assertTrue(whoami_email.startswith('branch@identi.ty'))
+        # Verify that the environment variable overrides the value 
+        # in the file
+        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
+        self.assertTrue(whoami_email.startswith('other@environ.ment'))
+        if bzr_email is not None:
+            os.environ['BZREMAIL'] = bzr_email
 
     def test_invalid_commands(self):
         self.runbzr("pants", retcode=1)
@@ -93,34 +104,34 @@
     def test_ignore_patterns(self):
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(list(b.unknowns()), [])
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
         self.assertEquals(list(b.unknowns()), [])
-        assert self.backtick('bzr unknowns') == ''
+        assert self.capture('unknowns') == ''
 
         file('foo.c', 'wt').write('int main() {}')
         self.assertEquals(list(b.unknowns()), ['foo.c'])
-        assert self.backtick('bzr unknowns') == 'foo.c\n'
+        assert self.capture('unknowns') == 'foo.c\n'
 
         self.runbzr(['add', 'foo.c'])
-        assert self.backtick('bzr unknowns') == ''
+        assert self.capture('unknowns') == ''
 
         # 'ignore' works when creating the .bzignore file
         file('foo.blah', 'wt').write('blah')
         self.assertEquals(list(b.unknowns()), ['foo.blah'])
         self.runbzr('ignore *.blah')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\n'
+        assert file('.bzrignore', 'rU').read() == '*.blah\n'
 
         # 'ignore' works when then .bzrignore file already exists
         file('garh', 'wt').write('garh')
         self.assertEquals(list(b.unknowns()), ['garh'])
-        assert self.backtick('bzr unknowns') == 'garh\n'
+        assert self.capture('unknowns') == 'garh\n'
         self.runbzr('ignore garh')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
+        assert file('.bzrignore', 'rU').read() == '*.blah\ngarh\n'
 
     def test_revert(self):
         self.runbzr('init')
@@ -147,17 +158,28 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
+        file('hello', 'wt').write('xyz')
+        self.runbzr('commit -m xyz hello')
+        self.runbzr('revert -r 1 hello')
+        self.check_file_contents('hello', 'foo')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'xyz')
+        os.chdir('revertdir')
+        self.runbzr('revert')
+        os.chdir('..')
+
+
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
         from bzrlib.branch import Branch
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['a', 'c', 'subdir/'])
-        self.run_bzr('add', self.test_dir)
-        self.run_bzr('mv', 'a', 'b')
-        self.run_bzr('mv', 'b', 'subdir')
-        self.run_bzr('mv', 'subdir/b', 'a')
-        self.run_bzr('mv', 'a', 'c', 'subdir')
-        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
+        self.run_bzr_captured(['add', self.test_dir])
+        self.run_bzr_captured(['mv', 'a', 'b'])
+        self.run_bzr_captured(['mv', 'b', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
+        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
 
 
     def test_main_version(self):
@@ -184,15 +206,23 @@
         test.runbzr('add goodbye')
         test.runbzr('commit -m setup goodbye')
 
-    def test_revert(self):
-        self.example_branch()
-        file('hello', 'wt').write('bar')
-        file('goodbye', 'wt').write('qux')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-        self.check_file_contents('goodbye', 'qux')
-        self.runbzr('revert')
-        self.check_file_contents('goodbye', 'baz')
+    def test_diff(self):
+        self.example_branch()
+        file('hello', 'wt').write('hello world!')
+        self.runbzr('commit -m fixing hello')
+        output = self.runbzr('diff -r 2..3', backtick=1)
+        self.assert_('\n+hello world!' in output)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1)
+        self.assert_('\n+baz' in output)
+
+    def test_branch(self):
+        """Branch from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        self.runbzr('branch a c -r 1')
 
     def test_merge(self):
         from bzrlib.branch import Branch
@@ -214,8 +244,8 @@
         self.runbzr('merge ../b')
         self.check_file_contents('goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
-        a = Branch('.')
-        b = Branch('../b')
+        a = Branch.open('.')
+        b = Branch.open('../b')
         a.get_revision_xml(b.last_patch())
         self.log('pending merges: %s', a.pending_merges())
         #        assert a.pending_merges() == [b.last_patch()], "Assertion %s %s" \
@@ -239,10 +269,12 @@
         self.runbzr('branch a b')
         os.chdir('b')
         self.runbzr('pull')
+        os.mkdir('subdir')
+        self.runbzr('add subdir')
         self.runbzr('commit -m blah --unchanged')
         os.chdir('../a')
-        a = Branch('.')
-        b = Branch('../b')
+        a = Branch.open('.')
+        b = Branch.open('../b')
         assert a.revision_history() == b.revision_history()[:-1]
         self.runbzr('pull ../b')
         assert a.revision_history() == b.revision_history()
@@ -253,31 +285,30 @@
         os.chdir('../a')
         self.runbzr('merge ../b')
         self.runbzr('commit -m blah4 --unchanged')
-        os.chdir('../b')
-        self.runbzr('pull ../a')
+        os.chdir('../b/subdir')
+        self.runbzr('pull ../../a')
         assert a.revision_history()[-1] == b.revision_history()[-1]
         
-
     def test_add_reports(self):
         """add command prints the names of added files."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
-
-        from cStringIO import StringIO
-        out = StringIO()
-
-        ret = self.apply_redirected(None, out, None,
-                                    run_bzr,
-                                    ['add'])
-        self.assertEquals(ret, 0)
-
+        out = self.run_bzr_captured(['add'], retcode = 0)[0]
         # the ordering is not defined at the moment
-        results = sorted(out.getvalue().rstrip('\n').split('\n'))
+        results = sorted(out.rstrip('\n').split('\n'))
         self.assertEquals(['added dir',
-                           'added dir/sub.txt',
+                           'added dir'+os.sep+'sub.txt',
                            'added top.txt',],
                           results)
 
+    def test_unknown_command(self):
+        """Handling of unknown command."""
+        out, err = self.run_bzr_captured(['fluffy-badger'],
+                                         retcode=1)
+        self.assertEquals(out, '')
+        err.index('unknown command')
+        
+
 
 class OldTests(ExternalBase):
     """old tests moved from ./testbzr."""
@@ -287,7 +318,7 @@
         from os.path import exists
 
         runbzr = self.runbzr
-        backtick = self.backtick
+        capture = self.capture
         progress = self.log
 
         progress("basic branch creation")
@@ -295,7 +326,7 @@
         chdir('branch1')
         runbzr('init')
 
-        self.assertEquals(backtick('bzr root').rstrip(),
+        self.assertEquals(capture('root').rstrip(),
                           os.path.join(self.test_dir, 'branch1'))
 
         progress("status of new file")
@@ -304,26 +335,25 @@
         f.write('hello world!\n')
         f.close()
 
-        out = backtick("bzr unknowns")
-        self.assertEquals(out, 'test.txt\n')
+        self.assertEquals(capture('unknowns'), 'test.txt\n')
 
-        out = backtick("bzr status")
+        out = capture("status")
         assert out == 'unknown:\n  test.txt\n'
 
-        out = backtick("bzr status --all")
+        out = capture("status --all")
         assert out == "unknown:\n  test.txt\n"
 
-        out = backtick("bzr status test.txt --all")
+        out = capture("status test.txt --all")
         assert out == "unknown:\n  test.txt\n"
 
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
 
-        out = backtick("bzr status test.txt")
+        out = capture("status test.txt")
         assert out == "unknown:\n  test.txt\n"
 
-        out = backtick("bzr status")
+        out = capture("status")
         assert out == ("unknown:\n"
                        "  test.txt\n"
                        "  test2.txt\n")
@@ -331,11 +361,11 @@
         os.unlink('test2.txt')
 
         progress("command aliases")
-        out = backtick("bzr st --all")
+        out = capture("st --all")
         assert out == ("unknown:\n"
                        "  test.txt\n")
 
-        out = backtick("bzr stat")
+        out = capture("stat")
         assert out == ("unknown:\n"
                        "  test.txt\n")
 
@@ -345,7 +375,7 @@
         runbzr("help commands")
         runbzr("help slartibartfast", 1)
 
-        out = backtick("bzr help ci")
+        out = capture("help ci")
         out.index('aliases: ')
 
         progress("can't rename unversioned file")
@@ -354,8 +384,8 @@
         progress("adding a file")
 
         runbzr("add test.txt")
-        assert backtick("bzr unknowns") == ''
-        assert backtick("bzr status --all") == ("added:\n"
+        assert capture("unknowns") == ''
+        assert capture("status --all") == ("added:\n"
                                                 "  test.txt\n")
 
         progress("rename newly-added file")
@@ -363,7 +393,7 @@
         assert os.path.exists("hello.txt")
         assert not os.path.exists("test.txt")
 
-        assert backtick("bzr revno") == '0\n'
+        assert capture("revno") == '0\n'
 
         progress("add first revision")
         runbzr(['commit', '-m', 'add first revision'])
@@ -377,7 +407,7 @@
         runbzr("add sub1")
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
-        assert backtick("bzr relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
+        assert capture("relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
 
         assert exists("sub2")
         assert exists("sub2/hello.txt")
@@ -398,15 +428,16 @@
         runbzr(['commit', '-m', 'rename nested subdirectories'])
 
         chdir('sub1/sub2')
-        self.assertEquals(backtick('bzr root')[:-1],
+        self.assertEquals(capture('root')[:-1],
                           os.path.join(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
         assert exists('./hello.txt')
-        assert backtick('bzr relpath hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
-        assert backtick('bzr relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assertEquals(capture('relpath hello.txt'),
+                          os.path.join('sub1', 'sub2', 'hello.txt') + '\n')
+        assert capture('relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        assert backtick('bzr relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        assert capture('relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
 
         runbzr('move sub2/hello.txt .')
         assert exists('hello.txt')
@@ -421,7 +452,7 @@
 
         runbzr('commit -F msg.tmp')
 
-        assert backtick('bzr revno') == '5\n'
+        assert capture('revno') == '5\n'
         runbzr('export -r 5 export-5.tmp')
         runbzr('export export.tmp')
 
@@ -429,11 +460,11 @@
         runbzr('log -v')
         runbzr('log -v --forward')
         runbzr('log -m', retcode=1)
-        log_out = backtick('bzr log -m commit')
+        log_out = capture('log -m commit')
         assert "this is my new commit" in log_out
         assert "rename nested" not in log_out
         assert 'revision-id' not in log_out
-        assert 'revision-id' in backtick('bzr log --show-ids -m commit')
+        assert 'revision-id' in capture('log --show-ids -m commit')
 
 
         progress("file with spaces in name")

=== added file 'bzrlib/selftest/test_bad_files.py'
--- bzrlib/selftest/test_bad_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_bad_files.py	2005-09-18 16:09:41 +0000
@@ -0,0 +1,93 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests being able to ignore mad filetypes.
+"""
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BadFileKindError
+import os
+
+def verify_status(tester, branch, value):
+    from bzrlib.status import show_status
+    from cStringIO import StringIO
+
+    tof = StringIO()
+    show_status(branch, to_file=tof)
+    tof.seek(0)
+    tester.assertEquals(tof.readlines(), value)
+
+
+class TestBadFiles(TestCaseInTempDir):
+    
+    def test_bad_files(self): 
+        """Test that bzr will ignore files it doesn't like"""
+        from bzrlib.commit import commit
+        from bzrlib.add import smart_add
+        from bzrlib.branch import Branch
+
+        b = Branch.initialize('.')
+
+        self.build_tree(['one', 'two', 'three'])
+        smart_add('.')
+        commit(b, "Commit one", rev_id="a@u-0-0")
+        self.build_tree(['four'])
+        smart_add('.')
+        commit(b, "Commit two", rev_id="a@u-0-1")
+        self.build_tree(['five'])
+        smart_add('.')
+        commit(b, "Commit three", rev_id="a@u-0-2")
+
+        # We should now have a few files, lets try to
+        # put some bogus stuff in the tree
+
+        # We can only continue if we have mkfifo
+        if not hasattr(os, 'mkfifo'):
+            return
+
+        # status with nothing
+        verify_status(self, b, [])
+
+        os.mkfifo('a-fifo')
+        self.build_tree(['six'])
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           '  six\n'
+                           ])
+        
+        # Make sure smart_add can handle having a bogus
+        # file in the way
+        smart_add('.')
+        verify_status(self, b,
+                          ['added:\n',
+                           '  six\n',
+                           'unknown:\n',
+                           '  a-fifo\n',
+                           ])
+        commit(b, "Commit four", rev_id="a@u-0-3")
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           ])
+
+        # We should raise an error if we are adding a bogus file
+        # Is there a way to test the actual error that should be raised?
+        self.run_bzr('add', 'a-fifo', retcode=1)
+

=== modified file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-09-01 06:33:48 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-09-26 08:56:15 +0000
@@ -1,20 +1,19 @@
 import os
+import shutil
+import tempfile
 import unittest
 
 from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
 from bzrlib.errors import NotBranchError, NotVersionedError
-
-
-import tempfile
-import shutil
 from bzrlib.inventory import InventoryEntry, RootEntry
-from bzrlib.osutils import file_kind
+from bzrlib.osutils import file_kind, rename
 from bzrlib import changeset
 from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
                                 BackupBeforeChange, PermissionsMerge)
 from bzrlib.changeset import Inventory, apply_changeset, invert_dict
 
+
 class FalseTree(object):
     def __init__(self, realtree):
         self._realtree = realtree
@@ -100,7 +99,7 @@
 
     def change_path(self, id, path):
         old_path = os.path.join(self.dir, self.inventory[id])
-        os.rename(old_path, self.abs_path(path))
+        rename(old_path, self.abs_path(path))
         self.inventory[id] = path
 
 
@@ -240,7 +239,7 @@
         os.chmod(tree.full_path(id), mode)
 
     def merge_changeset(self, merge_factory):
-        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
+        conflict_handler = changeset.ExceptionConflictHandler()
         return make_merge_changeset(self.cset, self.this, self.base,
                                     self.other, conflict_handler,
                                     merge_factory)
@@ -475,7 +474,7 @@
         from bzrlib.merge import merge
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
-        branch = Branch("original", init=True)
+        branch = Branch.initialize("original")
         smart_add_branch(branch, ["original"], True, add_reporter_null)
         branch.commit("start branch.", verbose=False)
         # Mary branches it.
@@ -487,7 +486,7 @@
         file.close()
         branch.commit("change file1")
         # Mary does too
-        mary_branch = Branch("mary")
+        mary_branch = Branch.open("mary")
         file = open("mary/file2", "wt")
         file.write("Mary\n")
         file.close()

=== modified file 'bzrlib/selftest/test_parent.py'
--- bzrlib/selftest/test_parent.py	2005-08-30 05:55:13 +0000
+++ bzrlib/selftest/test_parent.py	2005-09-09 11:37:44 +0000
@@ -24,13 +24,13 @@
 class TestParent(TestCaseInTempDir):
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(b.get_parent(), None)
         
     
     def test_set_get_parent(self):
         """Set and then re-get the parent"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         url = 'http://bazaar-ng.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEquals(b.get_parent(), url)
@@ -40,7 +40,7 @@
         from bzrlib.commands import run_bzr
 
         os.mkdir('from')
-        branch_from = Branch('from', init=True)
+        branch_from = Branch.initialize('from')
         file('from/foo', 'wt').write('contents of foo')
         branch_from.add('foo')
         branch_from.commit('initial commit')
@@ -48,7 +48,7 @@
         os.mkdir('to')
         copy_branch(branch_from, 'to', None)
 
-        branch_to = Branch('to')
+        branch_to = Branch.open('to')
         abspath = os.path.abspath('from')
         self.assertEquals(branch_to.get_parent(), abspath)
         

=== added file 'bzrlib/selftest/test_revision_info.py'
--- bzrlib/selftest/test_revision_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_revision_info.py	2005-09-19 06:05:19 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BzrCommandError, NoSuchRevision
+from bzrlib.branch import Branch
+from bzrlib.revisionspec import RevisionSpec
+
+class TestRevisionInfo(TestCaseInTempDir):
+    
+    def check_error(self, output, *args):
+        """Verify that the expected error matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args, retcode=1)[1], output)
+
+    def check_output(self, output, *args):
+        """Verify that the expected output matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args)[0], output)
+
+    def test_revision_info(self):
+        """Test that 'bzr revision-info' reports the correct thing.
+        """
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        # Make sure revision-info without any arguments throws an exception
+        self.check_error('bzr: ERROR: You must supply a revision identifier\n',
+                         'revision-info')
+
+        values = {
+            1:'   1 a@r-0-1\n',
+            2:'   2 a@r-0-2\n',
+            3:'   3 a@r-0-3\n'
+        }
+
+        # Check the results of just specifying a numeric revision
+        self.check_output(values[1], 'revision-info', '1')
+        self.check_output(values[2], 'revision-info', '2')
+        self.check_output(values[3], 'revision-info', '3')
+        self.check_output(values[1]+values[2], 'revision-info', '1', '2')
+        self.check_output(values[1]+values[2]+values[3], 'revision-info', '1', '2', '3')
+        self.check_output(values[2]+values[1], 'revision-info', '2', '1')
+
+        # Check as above, only using the '--revision' syntax
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', '1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', '2')
+        self.check_output('   3 a@r-0-3\n', 'revision-info', '-r', '3')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n', 'revision-info', '-r', '1..2')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n   3 a@r-0-3\n'
+                , 'revision-info', '-r', '1..2..3')
+        self.check_output('   2 a@r-0-2\n   1 a@r-0-1\n', 'revision-info', '-r', '2..1')
+
+        # Now try some more advanced revision specifications
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', 'revid:a@r-0-1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', 'revid:a@r-0-2')
+
+    def test_cat_revision(self):
+        """Test bzr cat-revision.
+        """
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        revs = {
+            1:b.get_revision_xml_file('a@r-0-1').read(),
+            2:b.get_revision_xml_file('a@r-0-2').read(),
+            3:b.get_revision_xml_file('a@r-0-3').read()
+        }
+
+        self.check_output(revs[1], 'cat-revision', 'a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', 'a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', 'a@r-0-3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', '1')
+        self.check_output(revs[2], 'cat-revision', '-r', '2')
+        self.check_output(revs[3], 'cat-revision', '-r', '3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', 'revid:a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', '-r', 'revid:a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', '-r', 'revid:a@r-0-3')
+

=== modified file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	2005-09-01 06:33:48 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-09-09 11:37:44 +0000
@@ -12,7 +12,7 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add((".",), recurse=True)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -22,7 +22,7 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add((".",), recurse=True)
         for path in paths:
@@ -35,7 +35,7 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add(("branch",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -51,8 +51,8 @@
                        "original/child/", "original/child/path")
         
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add((".",), True, add_reporter_null)
         for path in paths:
             self.assertNotEqual((path, branch.inventory.path2id(path)),
@@ -68,7 +68,7 @@
         from bzrlib.add import smart_add
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add(paths)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -81,7 +81,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, (".",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -91,7 +91,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         os.chdir("original")
         smart_add_branch(branch, (".",))
         for path in paths:
@@ -104,7 +104,7 @@
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch("branch", init=True)
+        branch = Branch.initialize("branch")
         smart_add_branch(branch, ("branch",))
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)
@@ -118,8 +118,8 @@
         build_paths = ("original/", "original/file1", "original/file2", 
                        "original/child/", "original/child/path")
         self.build_tree(build_paths)
-        branch = Branch(".", init=True)
-        child_branch = Branch("original/child", init=True)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
         smart_add_branch(branch, (".",))
         for path in paths:
             self.assertNotEqual((path, branch.inventory.path2id(path)),
@@ -135,7 +135,7 @@
         from bzrlib.add import smart_add_branch
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch(".", init=True)
+        branch = Branch.initialize(".")
         smart_add_branch(branch, paths)
         for path in paths:
             self.assertNotEqual(branch.inventory.path2id(path), None)

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-09-06 11:09:03 +0000
+++ bzrlib/selftest/testbranch.py	2005-09-14 05:22:13 +0000
@@ -21,7 +21,7 @@
     """Test appending more than one revision"""
     def test_append_revisions(self):
         from bzrlib.branch import Branch
-        br = Branch(".", init=True)
+        br = Branch.initialize(".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")

=== modified file 'bzrlib/selftest/testfetch.py'
--- bzrlib/selftest/testfetch.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/testfetch.py	2005-09-26 08:56:15 +0000
@@ -13,15 +13,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import sys
+
 import bzrlib.errors
 from bzrlib.selftest.testrevision import make_branches
 from bzrlib.trace import mutter
 from bzrlib.branch import Branch
-import sys
-import os
 
 from bzrlib.selftest import TestCaseInTempDir
-        
+
 
 class TestFetch(TestCaseInTempDir):
     def runTest(self):
@@ -29,7 +31,7 @@
 
         def new_branch(name):
             os.mkdir(name)
-            return Branch(name, init=True)
+            return Branch.initialize(name)
             
         #highest indices a: 5, b: 7
         br_a, br_b = make_branches()

=== modified file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	2005-09-10 23:46:35 +0000
+++ bzrlib/selftest/testgraph.py	2005-09-21 15:33:23 +0000
@@ -1,5 +1,5 @@
 from bzrlib.selftest import TestCase
-from bzrlib.graph import farthest_nodes
+from bzrlib.graph import node_distances, nodes_by_distance
 
 class TestBase(TestCase):
     def edge_add(self, *args):
@@ -18,19 +18,32 @@
         self.edge_add('A', 'G', 'H', 'I', 'B')
         self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
         self.edge_add('O', 'N')
-    
-    def test_farthest(self):
+
+    def node_descendants(self):
         descendants = {'A':set()}
         for node in self.graph:
             for ancestor in self.graph[node]:
                 if ancestor not in descendants:
                     descendants[ancestor] = set()
                 descendants[ancestor].add(node)
-        nodes = farthest_nodes(self.graph, descendants, 'A')
+        return descendants
+    
+    def test_distances(self):
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        nodes = nodes_by_distance(distances)
         self.assertEqual(nodes[0], 'D')
         assert nodes[1] in ('N', 'C')
         assert nodes[2] in ('N', 'C')
         assert nodes[3] in ('B', 'M')
         assert nodes[4] in ('B', 'M')
 
+        #Ensure we don't shortcut through B when there's only a difference of
+        # 1 in distance
+        self.graph = {}
+        self.edge_add('A', 'B', 'C')
+        self.edge_add('A', 'D', 'E', 'C')
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        self.assertEqual(distances['C'], 3)
 

=== modified file 'bzrlib/selftest/testhashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-08-30 03:10:32 +0000
+++ bzrlib/selftest/testhashcache.py	2005-09-23 09:25:16 +0000
@@ -14,6 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
+import time
 from bzrlib.selftest import TestCaseInTempDir
 
 
@@ -24,7 +26,9 @@
 
 
 def pause():
-    import time
+    if os.name == 'nt':
+        time.sleep(3)
+        return
     # allow it to stabilize
     start = int(time.time())
     while int(time.time()) == start:

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-09-05 08:00:35 +0000
+++ bzrlib/selftest/testinv.py	2005-09-23 09:25:16 +0000
@@ -14,6 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
 from bzrlib.selftest import TestCase
 
 from bzrlib.inventory import Inventory, InventoryEntry
@@ -23,9 +24,10 @@
 
     def test_is_within(self):
         from bzrlib.osutils import is_inside_any
-        
-        for dirs, fn in [(['src', 'doc'], 'src/foo.c'),
-                         (['src'], 'src/foo.c'),
+
+        SRC_FOO_C = os.path.join('src', 'foo.c')
+        for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
+                         (['src'], SRC_FOO_C),
                          (['src'], 'src'),
                          ]:
             self.assert_(is_inside_any(dirs, fn))

=== modified file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	2005-09-05 01:31:37 +0000
+++ bzrlib/selftest/testlog.py	2005-09-23 03:05:48 +0000
@@ -65,7 +65,7 @@
             self.assertEquals(expected, got)
 
     def test_cur_revno(self):
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         lf = LogCatcher()
         b.commit('empty commit')
@@ -86,7 +86,7 @@
     def test_simple_log(self):
         eq = self.assertEquals
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         lf = LogCatcher()
         show_log(b, lf)
@@ -132,3 +132,12 @@
         self.log('log 2 delta: %r' % d)
         # self.checkDelta(d, added=['hello'])
         
+        # commit a log message with control characters
+        msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
+        b.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg != committed_msg)
+        self.assert_(len(committed_msg) > len(msg))

=== modified file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	2005-09-14 16:35:46 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-26 08:56:15 +0000
@@ -1,13 +1,17 @@
+import os
+
 from bzrlib.branch import Branch
 from bzrlib.commit import commit
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.merge import merge
 from bzrlib.errors import UnrelatedBranches, NoCommits
-import os
+from bzrlib.revision import common_ancestor
+
+
 class TestMerge(TestCaseInTempDir):
     """Test appending more than one revision"""
     def test_pending(self):
-        br = Branch(".", init=True)
+        br = Branch.initialize(".")
         commit(br, "lala!")
         self.assertEquals(len(br.pending_merges()), 0)
         merge(['.', -1], [None, None])
@@ -16,7 +20,7 @@
     def test_nocommits(self):
         self.test_pending()
         os.mkdir('branch2')
-        br2 = Branch('branch2', init=True)
+        br2 = Branch.initialize('branch2')
         self.assertRaises(NoCommits, merge, ['branch2', -1], 
                           [None, None])
         return br2
@@ -24,7 +28,21 @@
     def test_unrelated(self):
         br2 = self.test_nocommits()
         commit(br2, "blah")
-        merge, ['branch2', -1], [None, None]
         self.assertRaises(UnrelatedBranches, merge, ['branch2', -1], 
                           [None, None])
-
+        return br2
+
+    def test_pending_with_null(self):
+        """When base is forced to revno 0, pending_merges is set"""
+        br2 = self.test_unrelated()
+        br1 = Branch.open('.')
+        merge(['branch2', -1], ['branch2', 0])
+        self.assertEquals(len(br1.pending_merges()), 1)
+        return (br1, br2)
+
+    def test_two_roots(self):
+        """Merge base is sane when two unrelated branches are merged"""
+        br1, br2 = self.test_pending_with_null()
+        commit(br1, "blah")
+        last = br1.last_patch()
+        self.assertEquals(common_ancestor(last, last, br1), last)

=== modified file 'bzrlib/selftest/testremotebranch.py'
--- bzrlib/selftest/testremotebranch.py	2005-09-14 18:07:16 +0000
+++ bzrlib/selftest/testremotebranch.py	2005-09-21 06:07:21 +0000
@@ -17,6 +17,7 @@
 from bzrlib.selftest.testrevision import make_branches
 from bzrlib.trace import mutter
 from bzrlib.branch import Branch, find_branch
+import gzip
 import sys
 import os
 
@@ -29,11 +30,25 @@
 
         def new_branch(name):
             os.mkdir(name)
-            return Branch(name, init=True)
+            return Branch.initialize(name)
             
         #highest indices a: 5, b: 7
         br_a, br_b = make_branches()
-        br_rem = find_branch(self.get_remote_url(br_a.base))
+        # unpack one of br_a's revision files to test .gz fallbacks
+        to_unzip = br_a.revision_history()[-1]
+        to_unzip_source = gzip.open(os.path.join(br_a.base, '.bzr', 
+                                                  'revision-store',
+                                                  to_unzip + '.gz'))
+        content = to_unzip_source.read()
+        to_unzip_source.close()
+        os.unlink(os.path.join(br_a.base, '.bzr', 'revision-store',
+                               to_unzip + '.gz'))
+        to_unzip_output = open(os.path.join(br_a.base, '.bzr', 
+                                             'revision-store', to_unzip), 'wb')
+        to_unzip_output.write(content)
+        to_unzip_output.close()
+        
+        br_rem = Branch.open(self.get_remote_url(br_a.base))
         assert not has_revision(br_b, br_rem.revision_history()[3])
         assert has_revision(br_b, br_rem.revision_history()[2])
         assert len(br_b.revision_history()) == 7

=== modified file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	2005-09-12 12:39:58 +0000
+++ bzrlib/selftest/testrevision.py	2005-09-23 03:22:16 +0000
@@ -15,21 +15,22 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from bzrlib.selftest import TestCaseInTempDir
-
+from bzrlib.revision import is_ancestor, MultipleRevisionSources
+from bzrlib.revision import combined_graph
 
 def make_branches():
     from bzrlib.branch import Branch
     from bzrlib.commit import commit
     import os
     os.mkdir("branch1")
-    br1 = Branch("branch1", init=True)
+    br1 = Branch.initialize("branch1")
     
     commit(br1, "Commit one", rev_id="a@u-0-0")
     commit(br1, "Commit two", rev_id="a@u-0-1")
     commit(br1, "Commit three", rev_id="a@u-0-2")
 
     os.mkdir("branch2")
-    br2 = Branch("branch2", init=True)
+    br2 = Branch.initialize("branch2")
     br2.update_revisions(br1)
     commit(br2, "Commit four", rev_id="b@u-0-3")
     commit(br2, "Commit five", rev_id="b@u-0-4")
@@ -48,7 +49,6 @@
 class TestIsAncestor(TestCaseInTempDir):
     def test_is_ancestor(self):
         """Test checking whether a revision is an ancestor of another revision"""
-        from bzrlib.revision import is_ancestor, MultipleRevisionSources
         from bzrlib.errors import NoSuchRevision
         br1, br2 = make_branches()
         revisions = br1.revision_history()
@@ -212,3 +212,15 @@
         self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
                           revisions[4])
 
+    def test_combined(self):
+        """combined_graph
+        Ensure it's not order-sensitive
+        """
+        br1, br2 = make_branches()
+        source = MultipleRevisionSources(br1, br2)
+        combined_1 = combined_graph(br1.last_patch(), br2.last_patch(), source)
+        combined_2 = combined_graph(br2.last_patch(), br1.last_patch(), source)
+        assert combined_1[1] == combined_2[1]
+        assert combined_1[2] == combined_2[2]
+        assert combined_1[3] == combined_2[3]
+        assert combined_1 == combined_2

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-14 17:23:14 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-23 08:54:16 +0000
@@ -15,47 +15,66 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import time
 from bzrlib.selftest import TestCaseInTempDir
 from bzrlib.errors import NoCommonAncestor, NoCommits
 from bzrlib.branch import copy_branch
 from bzrlib.merge import merge
 
 class TestRevisionNamespaces(TestCaseInTempDir):
+
     def test_revision_namespaces(self):
-        """Functional tests for hashcache"""
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
         from bzrlib.errors import NoSuchRevision
         from bzrlib.branch import Branch
-
-        b = Branch('.', init=True)
-
-        b.commit('Commit one', rev_id='a@r-0-1')
+        from bzrlib.revisionspec import RevisionSpec
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
-        self.assertEquals(b.get_revision_info(None), (0, None))
-        self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(NoSuchRevision, b.get_revision_info, 'revid:a@r-0-0')
-        self.assertRaises(TypeError, b.get_revision_info, object)
-
-        self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
+        self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revid:a@r-0-1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision,
+                          RevisionSpec('revid:a@r-0-0').in_history, b)
+        self.assertRaises(TypeError, RevisionSpec, object)
+
+        self.assertEquals(RevisionSpec('date:today').in_history(b),
+                          (2, 'a@r-0-2'))
+        self.assertEquals(RevisionSpec('date:yesterday').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('before:date:today').in_history(b),
+                          (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec('last:1').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('-1').in_history(b), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b).rev_id,
+                          'a@r-0-3')
 
         os.mkdir('newbranch')
-        b2 = Branch('newbranch', init=True)
-        self.assertEquals(b2.lookup_revision('revid:a@r-0-1'), 'a@r-0-1')
+        b2 = Branch.initialize('newbranch')
+        self.assertRaises(NoCommits, RevisionSpec('ancestor:.').in_history, b2)
 
-        self.assertRaises(NoCommits, b2.lookup_revision, 'ancestor:.')
-        self.assertEquals(b.lookup_revision('ancestor:.'), 'a@r-0-3')
         os.mkdir('copy')
         b3 = copy_branch(b, 'copy')
         b3.commit('Commit four', rev_id='b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'a@r-0-3')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'a@r-0-3')
         merge(['copy', -1], [None, None])
         b.commit('Commit five', rev_id='a@r-0-4')
-        self.assertEquals(b.lookup_revision('ancestor:copy'), 'b@r-0-4')
-        self.assertEquals(b3.lookup_revision('ancestor:.'), 'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
+                          'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'b@r-0-4')

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-09-04 02:59:56 +0000
+++ bzrlib/selftest/teststatus.py	2005-09-23 02:58:03 +0000
@@ -22,6 +22,7 @@
 
 
 from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.revisionspec import RevisionSpec
 
 class BranchStatus(TestCaseInTempDir):
     
@@ -31,7 +32,7 @@
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         # status with nothing
         tof = StringIO()
@@ -51,3 +52,44 @@
                            '  pending@pending-0-0\n'
                            ])
 
+    def test_branch_status_revisions(self):
+        """Tests branch status with revisions"""
+        from cStringIO import StringIO
+        from bzrlib.status import show_status
+        from bzrlib.branch import Branch
+        
+        b = Branch.initialize('.')
+
+        tof = StringIO()
+        self.build_tree(['hello.c', 'bye.c'])
+        b.add('hello.c')
+        b.add('bye.c')
+        b.commit('Test message')
+
+        tof = StringIO()
+        revs =[]
+        revs.append(RevisionSpec(0))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+        self.build_tree(['more.c'])
+        b.add('more.c')
+        b.commit('Another test message')
+        
+        tof = StringIO()
+        revs.append(RevisionSpec(1))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+

=== modified file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/teststore.py	2005-09-26 08:56:15 +0000
@@ -16,96 +16,21 @@
 
 """Test Store implementation
 """
-from bzrlib.transport.local import LocalTransport
-from bzrlib.transport import NoSuchFile
-from bzrlib.store.compressed_text import CompressedTextStore
+
+from StringIO import StringIO
+
+from bzrlib.store import ImmutableStore
 from bzrlib.selftest import TestCaseInTempDir
-from StringIO import StringIO
 from bzrlib.errors import BzrError
 
-def fill_store(store):
-    store.add(StringIO('hello'), 'a')
-    store.add(StringIO('other'), 'b')
-    store.add(StringIO('something'), 'c')
-    store.add(StringIO('goodbye'), '123123')
-
-def check_equals(tester, store, files, values, ignore_missing=False):
-    files = store.get(files, ignore_missing=ignore_missing)
-    count = 0
-    for f, v in zip(files, values):
-        count += 1
-        if v is None:
-            tester.assert_(f is None)
-        else:
-            tester.assertEquals(f.read(), v)
-    tester.assertEquals(count, len(values))
-    # We need to check to make sure there are no more
-    # files to be returned, I'm using a cheezy way
-    # Convert to a list, and there shouldn't be any left
-    tester.assertEquals(len(list(files)), 0)
-
-def test_multiple_add(tester, store):
-    fill_store(store)
-    tester.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
-
-def test_get(tester, store):
-    fill_store(store)
-
-    check_equals(tester, store, ['a'], ['hello'])
-    check_equals(tester, store, ['b', 'c'], ['other', 'something'])
-
-    # Make sure that requesting a non-existing file fails
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d'], [None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd'], ['hello', None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'a'], [None, 'hello'])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'd', 'd'], [None, None, None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd', 'b'], ['hello', None, 'other'])
-
-def test_ignore_get(tester, store):
-    fill_store(store)
-
-    files = store.get(['d'], ignore_missing=True)
-    files = list(files)
-    tester.assertEquals(len(files), 1)
-    tester.assert_(files[0] is None)
-
-    check_equals(tester, store, ['a', 'd'], ['hello', None],
-            ignore_missing=True)
-    check_equals(tester, store, ['d', 'a'], [None, 'hello'],
-            ignore_missing=True)
-    check_equals(tester, store, ['d', 'd'], [None, None],
-            ignore_missing=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            ignore_missing=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            ignore_missing=True)
-    check_equals(tester, store, ['b', 'd', 'c'], ['other', None, 'something'],
-            ignore_missing=True)
-
-def get_compressed_store():
-    t = LocalTransport('.')
-    return CompressedTextStore(t)
-
-class TestCompressedTextStore(TestCaseInTempDir):
+
+class TestStore(TestCaseInTempDir):
     def test_multiple_add(self):
         """Multiple add with same ID should raise a BzrError"""
-        store = get_compressed_store()
-        test_multiple_add(self, store)
-
-    def test_get(self):
-        store = get_compressed_store()
-        test_get(self, store)
-
-    def test_ignore_get(self):
-        store = get_compressed_store()
-        test_ignore_get(self, store)
-
+        store = ImmutableStore('.')
+        store.add(StringIO('goodbye'), '123123')
+        self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
 
 TEST_CLASSES = [
-    TestCompressedTextStore,
+    TestStore,
     ]

=== removed file 'bzrlib/selftest/testtransport.py'
--- bzrlib/selftest/testtransport.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/testtransport.py	1970-01-01 00:00:00 +0000
@@ -1,227 +0,0 @@
-# Copyright (C) 2004, 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-from bzrlib.selftest import TestCaseInTempDir
-
-def test_transport(tester, t, readonly=False):
-    """Test a transport object. Basically, it assumes that the
-    Transport object is connected to the current working directory.
-    So that whatever is done through the transport, should show
-    up in the working directory, and vice-versa.
-
-    This also tests to make sure that the functions work with both
-    generators and lists (assuming iter(list) is effectively a generator)
-    """
-    import tempfile, os
-    from bzrlib.transport.local import LocalTransport
-
-    # Test has
-    files = ['a', 'b', 'e', 'g']
-    tester.build_tree(files)
-    tester.assertEqual(t.has('a'), True)
-    tester.assertEqual(t.has('c'), False)
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, False, False, True, False, True, False])
-    tester.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
-            [True, True, False, False, True, False, True, False])
-
-    # Test get
-    tester.assertEqual(t.get('a').read(), open('a').read())
-    content_f = t.get_multi(files)
-    for path,f in zip(files, content_f):
-        tester.assertEqual(open(path).read(), f.read())
-
-    content_f = t.get_multi(iter(files))
-    for path,f in zip(files, content_f):
-        tester.assertEqual(open(path).read(), f.read())
-
-    tester.assertRaises(NoSuchFile, t.get, 'c')
-    try:
-        files = list(t.get_multi(['a', 'b', 'c']))
-    except NoSuchFile:
-        pass
-    else:
-        tester.fail('Failed to raise NoSuchFile for missing file in get_multi')
-    try:
-        files = list(t.get_multi(iter(['a', 'b', 'c', 'e'])))
-    except NoSuchFile:
-        pass
-    else:
-        tester.fail('Failed to raise NoSuchFile for missing file in get_multi')
-
-    # Test put
-    if readonly:
-        open('c', 'wb').write('some text for c\n')
-    else:
-        t.put('c', 'some text for c\n')
-    tester.assert_(os.path.exists('c'))
-    tester.assertEqual(open('c').read(), 'some text for c\n')
-    tester.assertEqual(t.get('c').read(), 'some text for c\n')
-    # Make sure 'has' is updated
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, True, False, True, False, True, False])
-    if readonly:
-        open('a', 'wb').write('new\ncontents for\na\n')
-        open('d', 'wb').write('contents\nfor d\n')
-    else:
-        # Put also replaces contents
-        tester.assertEqual(t.put_multi([('a', 'new\ncontents for\na\n'),
-                                      ('d', 'contents\nfor d\n')]),
-                         2)
-    tester.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-            [True, True, True, True, True, False, True, False])
-    tester.assertEqual(open('a').read(), 'new\ncontents for\na\n')
-    tester.assertEqual(open('d').read(), 'contents\nfor d\n')
-
-    if readonly:
-        open('a', 'wb').write('diff\ncontents for\na\n')
-        open('d', 'wb').write('another contents\nfor d\n')
-    else:
-        tester.assertEqual(
-            t.put_multi(iter([('a', 'diff\ncontents for\na\n'),
-                              ('d', 'another contents\nfor d\n')]))
-                         , 2)
-    tester.assertEqual(open('a').read(), 'diff\ncontents for\na\n')
-    tester.assertEqual(open('d').read(), 'another contents\nfor d\n')
-
-    if not readonly:
-        tester.assertRaises(NoSuchFile, t.put, 'path/doesnt/exist/c', 'contents')
-
-    # Test mkdir
-    os.mkdir('dir_a')
-    tester.assertEqual(t.has('dir_a'), True)
-    tester.assertEqual(t.has('dir_b'), False)
-
-    if readonly:
-        os.mkdir('dir_b')
-    else:
-        t.mkdir('dir_b')
-    tester.assertEqual(t.has('dir_b'), True)
-    tester.assert_(os.path.isdir('dir_b'))
-
-    if readonly:
-        os.mkdir('dir_c')
-        os.mkdir('dir_d')
-    else:
-        t.mkdir_multi(['dir_c', 'dir_d'])
-    tester.assertEqual(list(t.has_multi(['dir_a', 'dir_b', 'dir_c', 'dir_d', 'dir_e', 'dir_b'])),
-            [True, True, True, True, False, True])
-    for d in ['dir_a', 'dir_b', 'dir_c', 'dir_d']:
-        tester.assert_(os.path.isdir(d))
-
-    if not readonly:
-        tester.assertRaises(NoSuchFile, t.mkdir, 'path/doesnt/exist')
-        tester.assertRaises(FileExists, t.mkdir, 'dir_a') # Creating a directory again should fail
-
-    # Make sure the transport recognizes when a
-    # directory is created by other means
-    # Caching Transports will fail, because dir_e was already seen not
-    # to exist. So instead, we will search for a new directory
-    #os.mkdir('dir_e')
-    #if not readonly:
-    #    tester.assertRaises(FileExists, t.mkdir, 'dir_e')
-
-    os.mkdir('dir_f')
-    if not readonly:
-        tester.assertRaises(FileExists, t.mkdir, 'dir_f')
-
-    # Test get/put in sub-directories
-    if readonly:
-        open('dir_a/a', 'wb').write('contents of dir_a/a')
-        open('dir_b/b', 'wb').write('contents of dir_b/b')
-    else:
-        tester.assertEqual(
-            t.put_multi([('dir_a/a', 'contents of dir_a/a'),
-                         ('dir_b/b', 'contents of dir_b/b')])
-                      , 2)
-    for f in ('dir_a/a', 'dir_b/b'):
-        tester.assertEqual(t.get(f).read(), open(f).read())
-
-    # Test copy_to
-    dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
-    dtmp_base = os.path.basename(dtmp)
-    local_t = LocalTransport(dtmp)
-
-    files = ['a', 'b', 'c', 'd']
-    t.copy_to(files, local_t)
-    for f in files:
-        tester.assertEquals(open(f).read(), open(os.path.join(dtmp_base, f)).read())
-
-    # TODO: Test append
-    # TODO: Make sure all entries support file-like objects as well as strings.
-
-class LocalTransportTest(TestCaseInTempDir):
-    def test_local_transport(self):
-        from bzrlib.transport.local import LocalTransport
-
-        t = LocalTransport('.')
-        test_transport(self, t)
-
-class HttpServer(object):
-    """This just encapsulates spawning and stopping
-    an httpserver.
-    """
-    def __init__(self):
-        """This just spawns a separate process to serve files from
-        this directory. Call the .stop() function to kill the
-        process.
-        """
-        from BaseHTTPServer import HTTPServer
-        from SimpleHTTPServer import SimpleHTTPRequestHandler
-        import os
-        if hasattr(os, 'fork'):
-            self.pid = os.fork()
-            if self.pid != 0:
-                return
-        else: # How do we handle windows, which doesn't have fork?
-            raise NotImplementedError('At present HttpServer cannot fork on Windows')
-
-            # We might be able to do something like os.spawn() for the
-            # python executable, and give it a simple script to run.
-            # but then how do we kill it?
-
-        try:
-            self.s = HTTPServer(('', 9999), SimpleHTTPRequestHandler)
-            # TODO: Is there something nicer than killing the server when done?
-            self.s.serve_forever()
-        except KeyboardInterrupt:
-            pass
-        os._exit(0)
-
-    def stop(self):
-        import os
-        if self.pid is None:
-            return
-        if hasattr(os, 'kill'):
-            import signal
-            os.kill(self.pid, signal.SIGINT)
-            os.waitpid(self.pid, 0)
-            self.pid = None
-        else:
-            raise NotImplementedError('At present HttpServer cannot stop on Windows')
-
-class HttpTransportTest(TestCaseInTempDir):
-    def test_http_transport(self):
-        from bzrlib.transport.http import HttpTransport
-
-        s = HttpServer()
-
-        t = HttpTransport('http://localhost:9999/')
-        test_transport(self, t, readonly=True)
-
-        s.stop()
-

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-09-12 01:29:49 +0000
+++ bzrlib/selftest/versioning.py	2005-09-26 08:56:15 +0000
@@ -44,7 +44,7 @@
 
         from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
-        b = Branch('.')
+        b = Branch.open('.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
@@ -63,7 +63,7 @@
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['foo/',
                          'foo/hello'])
@@ -84,7 +84,7 @@
         from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
@@ -112,12 +112,12 @@
         from bzrlib.commands import run_bzr
         eq = self.assertEqual
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(b.unknowns()), ['inertiatic'])
         run_bzr(['add', '--no-recurse', 'inertiatic'])
-        eq(list(b.unknowns()), ['inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'+os.sep+'esp'])
         run_bzr(['add', 'inertiatic/esp'])
         eq(list(b.unknowns()), [])
 
@@ -131,7 +131,7 @@
         ass = self.assert_
         chdir = os.chdir
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['src/', 'README'])
         
         eq(sorted(b.unknowns()),
@@ -159,7 +159,7 @@
         """After all the above changes, run the check and upgrade commands.
 
         The upgrade should be a no-op."""
-        b = Branch('.')
+        b = Branch.open('.')
         debug('branch has %d revisions', b.revno())
         
         debug('check branch...')
@@ -186,7 +186,7 @@
         self.build_tree(['a/', 'b/'])
         
         run_bzr('init')
-        b = Branch('.')
+        b = Branch.open('.')
         
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('old contents')
@@ -199,8 +199,8 @@
             
         run_bzr('commit', 'a', '-m', 'commit a only')
         
-        old = b.revision_tree(b.lookup_revision(1))
-        new = b.revision_tree(b.lookup_revision(2))
+        old = b.revision_tree(b.get_rev_id(1))
+        new = b.revision_tree(b.get_rev_id(2))
         
         eq(new.get_file_by_path('b/two').read(), 'old contents')
         eq(new.get_file_by_path('top').read(), 'old contents')
@@ -209,14 +209,14 @@
         os.chdir('a')
         # commit from here should do nothing
         run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
-        v3 = b.revision_tree(b.lookup_revision(3))
+        v3 = b.revision_tree(b.get_rev_id(3))
         eq(v3.get_file_by_path('b/two').read(), 'old contents')
         eq(v3.get_file_by_path('top').read(), 'old contents')
         eq(v3.get_file_by_path('a/one').read(), 'new contents')
                 
         # commit in subdirectory commits whole tree
         run_bzr('commit', '-m', 'commit whole tree from subdir')
-        v4 = b.revision_tree(b.lookup_revision(4))
+        v4 = b.revision_tree(b.get_rev_id(4))
         eq(v4.get_file_by_path('b/two').read(), 'new contents')        
         eq(v4.get_file_by_path('top').read(), 'new contents')
         

=== modified file 'bzrlib/selftest/whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-09-15 21:35:53 +0000
+++ bzrlib/selftest/whitebox.py	2005-09-23 09:25:16 +0000
@@ -9,7 +9,7 @@
 class TestBranch(TestCaseInTempDir):
 
     def test_unknowns(self):
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt',
                          'hello.txt~'])
@@ -20,7 +20,7 @@
     def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
 
@@ -74,7 +74,7 @@
 
     def test_pending_merges(self):
         """Tracking pending-merged revisions."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.assertEquals(b.pending_merges(), [])
         b.add_pending_merge('foo@azkhazan-123123-abcabc')
@@ -97,7 +97,7 @@
         
     def test_revert(self):
         """Test selected-file revert"""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
@@ -124,7 +124,7 @@
 
     def test_rename_dirs(self):
         """Test renaming directories and the files within them."""
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
         b.add(['dir', 'dir/sub', 'dir/sub/file'])
 
@@ -195,7 +195,8 @@
             # directory, or nearby
             os.chdir(dtmp)
 
-            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
+            FOO_BAR_QUUX = os.path.join('foo', 'bar', 'quux')
+            self.assertEqual(rp('foo/bar/quux'), FOO_BAR_QUUX)
 
             self.assertEqual(rp('foo'), 'foo')
 

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-09-07 23:14:30 +0000
+++ bzrlib/status.py	2005-09-23 02:58:03 +0000
@@ -20,7 +20,8 @@
                 specific_files=None,
                 show_ids=False,
                 to_file=None,
-                show_pending=True):
+                show_pending=True,
+                revision=None):
     """Display status for non-ignored working files.
 
     show_unchanged
@@ -37,6 +38,10 @@
 
     show_pending
         If set, write pending merges.
+    revision
+        If None the compare latest revision with working tree
+        If one revision show compared it with working tree.
+        If two revisions show status between first and second.
     """
     import sys
     from bzrlib.delta import compare_trees
@@ -46,9 +51,26 @@
     
     branch.lock_read()
     try:
-
-        old = branch.basis_tree()
-        new = branch.working_tree()
+        new_is_working_tree = True
+        if revision is None:
+            old = branch.basis_tree()
+            new = branch.working_tree()
+        elif len(revision) > 0:
+            try:
+                rev_id = revision[0].in_history(branch).rev_id
+                old = branch.revision_tree(rev_id)
+            except NoSuchRevision, e:
+                raise BzrCommandError(str(e))
+            if len(revision) > 1:
+                try:
+                    rev_id = revision[1].in_history(branch).rev_id
+                    new = branch.revision_tree(rev_id)
+                    new_is_working_tree = False
+                except NoSuchRevision, e:
+                    raise BzrCommandError(str(e))
+            else:
+                new = branch.working_tree()
+                
 
         delta = compare_trees(old, new, want_unchanged=show_unchanged,
                               specific_files=specific_files)
@@ -57,22 +79,23 @@
                    show_ids=show_ids,
                    show_unchanged=show_unchanged)
 
-        unknowns = new.unknowns()
-        done_header = False
-        for path in unknowns:
-            # FIXME: Should also match if the unknown file is within a
-            # specified directory.
-            if specific_files:
-                if path not in specific_files:
-                    continue
-            if not done_header:
-                print >>to_file, 'unknown:'
-                done_header = True
-            print >>to_file, ' ', path
-        if show_pending and len(branch.pending_merges()) > 0:
-            print >>to_file, 'pending merges:'
-            for merge in branch.pending_merges():
-                print >> to_file, ' ', merge
+        if new_is_working_tree:
+            unknowns = new.unknowns()
+            done_header = False
+            for path in unknowns:
+                # FIXME: Should also match if the unknown file is within a
+                # specified directory.
+                if specific_files:
+                    if path not in specific_files:
+                        continue
+                if not done_header:
+                    print >>to_file, 'unknown:'
+                    done_header = True
+                print >>to_file, ' ', path
+            if show_pending and len(branch.pending_merges()) > 0:
+                print >>to_file, 'pending merges:'
+                for merge in branch.pending_merges():
+                    print >> to_file, ' ', merge
     finally:
         branch.unlock()
         

=== removed directory 'bzrlib/store'
=== renamed file 'bzrlib/store/__init__.py' => 'bzrlib/store.py'
--- bzrlib/store/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/store.py	2005-09-26 08:56:15 +0000
@@ -21,10 +21,19 @@
 unique ID.
 """
 
-import bzrlib
+import errno
+import gzip
+import os
+import tempfile
+import types
+from stat import ST_SIZE
+from StringIO import StringIO
+
+from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
 import bzrlib.ui
-import bzrlib.transport
+import bzrlib.osutils as osutils
+
 
 ######################################################################
 # stores
@@ -32,154 +41,223 @@
 class StoreError(Exception):
     pass
 
-class Store(object):
-    """This class represents the abstract storage layout for saving information.
+
+class ImmutableStore(object):
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    >>> st = ImmutableScratchStore()
+
+    >>> st.add(StringIO('hello'), 'aa')
+    >>> 'aa' in st
+    True
+    >>> 'foo' in st
+    False
+
+    You are not allowed to add an id that is already present.
+
+    Entries can be retrieved as files, which may then be read.
+
+    >>> st.add(StringIO('goodbye'), '123123')
+    >>> st['123123'].read()
+    'goodbye'
+
+    TODO: Atomic add by writing to a temporary file and renaming.
+
+    In bzr 0.0.5 and earlier, files within the store were marked
+    readonly on disk.  This is no longer done but existing stores need
+    to be accomodated.
     """
-    _transport = None
-    _max_buffered_requests = 10
-
-    def __init__(self, transport):
-        assert isinstance(transport, bzrlib.transport.Transport)
-        self._transport = transport
+
+    def __init__(self, basedir):
+        self._basedir = basedir
+
+    def _path(self, entry_id):
+        if not isinstance(entry_id, basestring):
+            raise TypeError(type(entry_id))
+        if '\\' in entry_id or '/' in entry_id:
+            raise ValueError("invalid store id %r" % entry_id)
+        return os.path.join(self._basedir, entry_id)
 
     def __repr__(self):
-        if self._transport is None:
-            return "%s(None)" % (self.__class__.__name__)
+        return "%s(%r)" % (self.__class__.__name__, self._basedir)
+
+    def add(self, f, fileid, compressed=True):
+        """Add contents of a file into the store.
+
+        f -- An open file, or file-like object."""
+        # FIXME: Only works on files that will fit in memory
+        
+        from bzrlib.atomicfile import AtomicFile
+        
+        mutter("add store entry %r" % (fileid))
+        if isinstance(f, types.StringTypes):
+            content = f
         else:
-            return "%s(%r)" % (self.__class__.__name__, self._transport.base)
-
-    __str__ = __repr__
-
-    def __len__(self):
-        raise NotImplementedError('Children should define their length')
-
-    def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
-        raise NotImplementedError
-
-    def __contains__(self, fileid):
-        """"""
-        raise NotImplementedError
-
-    def __iter__(self):
-        raise NotImplementedError
-
-    def add(self, f, fileid):
-        """Add a file object f to the store accessible from the given fileid"""
-        raise NotImplementedError('Children of Store must define their method of adding entries.')
-
-    def add_multi(self, entries):
-        """Add a series of file-like or string objects to the store with the given
-        identities.
-        
-        :param entries: A list of tuples of file,id pairs [(file1, id1), (file2, id2), ...]
-                        This could also be a generator yielding (file,id) pairs.
-        """
-        for f, fileid in entries:
-            self.add(f, fileid)
-
-    def has(self, fileids):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        for fileid in fileids:
-            if fileid in self:
-                yield True
+            content = f.read()
+            
+        p = self._path(fileid)
+        if os.access(p, os.F_OK) or os.access(p + '.gz', os.F_OK):
+            raise BzrError("store %r already contains id %r" % (self._basedir, fileid))
+
+        fn = p
+        if compressed:
+            fn = fn + '.gz'
+            
+        af = AtomicFile(fn, 'wb')
+        try:
+            if compressed:
+                gf = gzip.GzipFile(mode='wb', fileobj=af)
+                gf.write(content)
+                gf.close()
             else:
-                yield False
-
-    def get(self, fileids, ignore_missing=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        :param ignore_missing: If true, return None for entries which do not 
-                               exist.
-        :return: A list or generator of file-like objects, one for each id.
-        """
-        for fileid in fileids:
-            try:
-                yield self[fileid]
-            except KeyError:
-                if ignore_missing:
-                    yield None
-                else:
-                    raise
-
-    def copy_multi(self, other, ids, pb=None, permit_failure=False):
+                af.write(content)
+            af.commit()
+        finally:
+            af.close()
+
+
+    def copy_multi(self, other, ids, permit_failure=False):
         """Copy texts for ids from other into self.
 
-        If an id is present in self, it is skipped.  A count of copied
-        ids is returned, which may be less than len(ids).
+        If an id is present in self, it is skipped.
 
-        :param other: Another Store object
-        :param ids: A list of entry ids to be copied
-        :param pb: A ProgressBar object, if none is given, the default will be created.
-        :param permit_failure: Allow missing entries to be ignored
-        :return: (n_copied, [failed]) The number of entries copied successfully,
-            followed by a list of entries which could not be copied (because they
-            were missing)
+        Returns (count_copied, failed), where failed is a collection of ids
+        that could not be copied.
         """
-        if pb is None:
-            pb = bzrlib.ui.ui_factory.progress_bar()
-
-        ids = list(ids) # Make sure we don't have a generator, since we iterate 2 times
+        pb = bzrlib.ui.ui_factory.progress_bar()
+        
         pb.update('preparing to copy')
-        to_copy = []
-        for file_id, has in zip(ids, self.has(ids)):
-            if not has:
-                to_copy.append(file_id)
-        return self._do_copy(other, to_copy, pb, permit_failure=permit_failure)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        """This is the standard copying mechanism, just get them one at
-        a time from remote, and store them locally.
-
-        :param other: Another Store object
-        :param to_copy: A list of entry ids to copy
-        :param pb: A ProgressBar object to display completion status.
-        :param permit_failure: Allow missing entries to be ignored
-        :return: (n_copied, [failed])
-            The number of entries copied, and a list of failed entries.
-        """
-        # This should be updated to use add_multi() rather than
-        # the current methods of buffering requests.
-        # One question, is it faster to queue up 1-10 and then copy 1-10
-        # then queue up 11-20, copy 11-20
-        # or to queue up 1-10, copy 1, queue 11, copy 2, etc?
-        # sort of pipeline versus batch.
-
-        # We can't use self._transport.copy_to because we don't know
-        # whether the local tree is in the same format as other
+        to_copy = [id for id in ids if id not in self]
+        if isinstance(other, ImmutableStore):
+            return self.copy_multi_immutable(other, to_copy, pb, 
+                                             permit_failure=permit_failure)
+        count = 0
         failed = set()
-        def buffer_requests():
-            count = 0
-            buffered_requests = []
-            for fileid in to_copy:
+        for id in to_copy:
+            count += 1
+            pb.update('copy', count, len(to_copy))
+            if not permit_failure:
+                self.add(other[id], id)
+            else:
                 try:
-                    f = other[fileid]
+                    entry = other[id]
                 except KeyError:
-                    if permit_failure:
-                        failed.add(fileid)
-                        continue
+                    failed.add(id)
+                    continue
+                self.add(entry, id)
+                
+        if not permit_failure:
+            assert count == len(to_copy)
+        pb.clear()
+        return count, failed
+
+    def copy_multi_immutable(self, other, to_copy, pb, permit_failure=False):
+        from shutil import copyfile
+        count = 0
+        failed = set()
+        for id in to_copy:
+            p = self._path(id)
+            other_p = other._path(id)
+            try:
+                copyfile(other_p, p)
+            except IOError, e:
+                if e.errno == errno.ENOENT:
+                    if not permit_failure:
+                        copyfile(other_p+".gz", p+".gz")
                     else:
-                        raise
-
-                buffered_requests.append((f, fileid))
-                if len(buffered_requests) > self._max_buffered_requests:
-                    yield buffered_requests.pop(0)
-                    count += 1
-                    pb.update('copy', count, len(to_copy))
-
-            for req in buffered_requests:
-                yield req
-                count += 1
-                pb.update('copy', count, len(to_copy))
-
-            assert count == len(to_copy)
-
-        self.add_multi(buffer_requests())
-
+                        try:
+                            copyfile(other_p+".gz", p+".gz")
+                        except IOError, e:
+                            if e.errno == errno.ENOENT:
+                                failed.add(id)
+                            else:
+                                raise
+                else:
+                    raise
+            
+            count += 1
+            pb.update('copy', count, len(to_copy))
+        assert count == len(to_copy)
         pb.clear()
-        return len(to_copy), failed
-
+        return count, failed
+    
+
+    def __contains__(self, fileid):
+        """"""
+        p = self._path(fileid)
+        return (os.access(p, os.R_OK)
+                or os.access(p + '.gz', os.R_OK))
+
+    # TODO: Guard against the same thing being stored twice, compressed and uncompresse
+
+    def __iter__(self):
+        for f in os.listdir(self._basedir):
+            if f[-3:] == '.gz':
+                # TODO: case-insensitive?
+                yield f[:-3]
+            else:
+                yield f
+
+    def __len__(self):
+        return len(os.listdir(self._basedir))
+
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        p = self._path(fileid)
+        try:
+            return gzip.GzipFile(p + '.gz', 'rb')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+        try:
+            return file(p, 'rb')
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+
+        raise KeyError(fileid)
+
+
+    def total_size(self):
+        """Return (count, bytes)
+
+        This is the (compressed) size stored on disk, not the size of
+        the content."""
+        total = 0
+        count = 0
+        for fid in self:
+            count += 1
+            p = self._path(fid)
+            try:
+                total += os.stat(p)[ST_SIZE]
+            except OSError:
+                total += os.stat(p + '.gz')[ST_SIZE]
+                
+        return count, total
+
+
+
+
+class ImmutableScratchStore(ImmutableStore):
+    """Self-destructing test subclass of ImmutableStore.
+
+    The Store only exists for the lifetime of the Python object.
+ Obviously you should not put anything precious in it.
+    """
+    def __init__(self):
+        ImmutableStore.__init__(self, tempfile.mkdtemp())
+
+    def __del__(self):
+        for f in os.listdir(self._basedir):
+            fpath = os.path.join(self._basedir, f)
+            # needed on windows, and maybe some other filesystems
+            os.chmod(fpath, 0600)
+            os.remove(fpath)
+        os.rmdir(self._basedir)
+        mutter("%r destroyed" % self)

=== removed file 'bzrlib/store/compressed_text.py'
--- bzrlib/store/compressed_text.py	2005-09-15 21:35:53 +0000
+++ bzrlib/store/compressed_text.py	1970-01-01 00:00:00 +0000
@@ -1,255 +0,0 @@
-# Copyright (C) 2005 by Canonical Development Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""
-An implementation the primary storage type CompressedTextStore.
-
-This store keeps compressed versions of the full text. It does not
-do any sort of delta compression.
-"""
-
-import os, tempfile, gzip
-
-import bzrlib.store
-from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
-
-from StringIO import StringIO
-from stat import ST_SIZE
-
-class CompressedTextStore(bzrlib.store.Store):
-    """Store that holds files indexed by unique names.
-
-    Files can be added, but not modified once they are in.  Typically
-    the hash is used as the name, or something else known to be unique,
-    such as a UUID.
-
-    Files are stored gzip compressed, with no delta compression.
-
-    >>> st = ScratchCompressedTextStore()
-
-    >>> st.add(StringIO('hello'), 'aa')
-    >>> 'aa' in st
-    True
-    >>> 'foo' in st
-    False
-
-    You are not allowed to add an id that is already present.
-
-    Entries can be retrieved as files, which may then be read.
-
-    >>> st.add(StringIO('goodbye'), '123123')
-    >>> st['123123'].read()
-    'goodbye'
-    """
-
-    def __init__(self, basedir):
-        super(CompressedTextStore, self).__init__(basedir)
-
-    def _check_fileid(self, fileid):
-        if '\\' in fileid or '/' in fileid:
-            raise ValueError("invalid store id %r" % fileid)
-
-    def _relpath(self, fileid):
-        self._check_fileid(fileid)
-        return fileid + '.gz'
-
-    def add(self, f, fileid):
-        """Add contents of a file into the store.
-
-        f -- An open file, or file-like object."""
-        # TODO: implement an add_multi which can do some of it's
-        #       own piplelining, and possible take advantage of
-        #       transport.put_multi(). The problem is that
-        #       entries potentially need to be compressed as they
-        #       are received, which implies translation, which
-        #       means it isn't as straightforward as we would like.
-        from cStringIO import StringIO
-        from bzrlib.osutils import pumpfile
-        
-        mutter("add store entry %r" % (fileid))
-        if isinstance(f, basestring):
-            f = StringIO(f)
-            
-        fn = self._relpath(fileid)
-        if self._transport.has(fn):
-            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
-
-
-        sio = StringIO()
-        gf = gzip.GzipFile(mode='wb', fileobj=sio)
-        # if pumpfile handles files that don't fit in ram,
-        # so will this function
-        if isinstance(f, basestring):
-            gf.write(f)
-        else:
-            pumpfile(f, gf)
-        gf.close()
-        sio.seek(0)
-        self._transport.put(fn, sio)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        if isinstance(other, CompressedTextStore):
-            return self._copy_multi_text(other, to_copy, pb,
-                    permit_failure=permit_failure)
-        return super(CompressedTextStore, self)._do_copy(other, to_copy,
-                pb, permit_failure=permit_failure)
-
-    def _copy_multi_text(self, other, to_copy, pb,
-            permit_failure=False):
-        # Because of _transport, we can no longer assume
-        # that they are on the same filesystem, we can, however
-        # assume that we only need to copy the exact bytes,
-        # we don't need to process the files.
-
-        failed = set()
-        if permit_failure:
-            new_to_copy = set()
-            for fileid, has in zip(to_copy, self.has(to_copy)):
-                if has:
-                    new_to_copy.add(fileid)
-                else:
-                    failed.add(fileid)
-            to_copy = new_to_copy
-
-        paths = [self._relpath(fileid) for fileid in to_copy]
-        count = other._transport.copy_to(paths, self._transport, pb=pb)
-        assert count == len(to_copy)
-        return count, failed
-
-    def __contains__(self, fileid):
-        """"""
-        fn = self._relpath(fileid)
-        return self._transport.has(fn)
-
-    def has(self, fileids, pb=None):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        relpaths = (self._relpath(fid) for fid in fileids)
-        return self._transport.has_multi(relpaths, pb=pb)
-
-    def get(self, fileids, ignore_missing=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        TODO: Write some tests to make sure that ignore_missing is
-              handled correctly.
-
-        TODO: What should the exception be for a missing file?
-              KeyError, or NoSuchFile?
-        """
-
-        # This next code gets a bit hairy because it can allow
-        # to not request a file which doesn't seem to exist.
-        # Also, the same fileid may be requested twice, so we
-        # can't just build up a map.
-        rel_paths = [self._relpath(fid) for fid in fileids]
-        is_requested = []
-
-        if ignore_missing:
-            existing_paths = []
-            for path, has in zip(rel_paths,
-                    self._transport.has_multi(rel_paths)):
-                if has:
-                    existing_paths.append(path)
-                    is_requested.append(True)
-                else:
-                    is_requested.append(False)
-        else:
-            existing_paths = rel_paths
-            is_requested = [True for x in rel_paths]
-
-        count = 0
-        for f in self._transport.get_multi(existing_paths, pb=pb):
-            assert count < len(is_requested)
-            while not is_requested[count]:
-                yield None
-                count += 1
-            if hasattr(f, 'tell'):
-                yield gzip.GzipFile(mode='rb', fileobj=f)
-            else:
-                from cStringIO import StringIO
-                sio = StringIO(f.read())
-                yield gzip.GzipFile(mode='rb', fileobj=sio)
-            count += 1
-
-        while count < len(is_requested):
-            yield None
-            count += 1
-
-    def __iter__(self):
-        # TODO: case-insensitive?
-        for f in self._transport.list_dir('.'):
-            if f[-3:] == '.gz':
-                yield f[:-3]
-            else:
-                yield f
-
-    def __len__(self):
-        return len([f for f in self._transport.list_dir('.')])
-
-
-    def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
-        fn = self._relpath(fileid)
-        # This will throw if the file doesn't exist.
-        try:
-            f = self._transport.get(fn)
-        except:
-            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
-
-        # gzip.GzipFile.read() requires a tell() function
-        # but some transports return objects that cannot seek
-        # so buffer them in a StringIO instead
-        if hasattr(f, 'tell'):
-            return gzip.GzipFile(mode='rb', fileobj=f)
-        else:
-            from cStringIO import StringIO
-            sio = StringIO(f.read())
-            return gzip.GzipFile(mode='rb', fileobj=sio)
-            
-
-    def total_size(self):
-        """Return (count, bytes)
-
-        This is the (compressed) size stored on disk, not the size of
-        the content."""
-        total = 0
-        count = 0
-        relpaths = [self._relpath(fid) for fid in self]
-        for st in self._transport.stat_multi(relpaths):
-            count += 1
-            total += st[ST_SIZE]
-                
-        return count, total
-
-class ScratchCompressedTextStore(CompressedTextStore):
-    """Self-destructing test subclass of CompressedTextStore.
-
-    The Store only exists for the lifetime of the Python object.
-    Obviously you should not put anything precious in it.
-    """
-    def __init__(self):
-        from transport import transport
-        super(ScratchCompressedTextStore, self).__init__(transport(tempfile.mkdtemp()))
-
-    def __del__(self):
-        self._transport.delete_multi(self._transport.list_dir('.'))
-        os.rmdir(self._transport.base)
-        mutter("%r destroyed" % self)
-

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2005-04-11 02:44:45 +0000
+++ bzrlib/textinv.py	2005-09-26 08:56:15 +0000
@@ -15,8 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from errors import BzrError
-from inventory import InventoryEntry, Inventory
+from bzrlib.errors import BzrError
+from bzrlib.inventory import InventoryEntry, Inventory
 
 
 START_MARK = "# bzr inventory format 3\n"

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-08-29 10:57:01 +0000
+++ bzrlib/trace.py	2005-09-23 09:25:16 +0000
@@ -101,17 +101,9 @@
             return
         old_fname = trace_fname + '.old'
 
-        try:
-            # must remove before rename on windows
-            os.remove(old_fname)
-        except OSError:
-            pass
+        from osutils import rename
+        rename(trace_fname, old_fname)
 
-        try:
-            # might fail if in use on windows
-            os.rename(trace_fname, old_fname)
-        except OSError:
-            pass
     except OSError:
         return
 
@@ -197,7 +189,6 @@
     logging.getLogger('').addHandler(_stderr_handler)
 
 
-
 def disable_default_logging():
     """Turn off default log handlers.
 

=== removed directory 'bzrlib/transport'
=== removed file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,442 +0,0 @@
-#!/usr/bin/env python
-"""\
-This module contains the basic class handling transport of
-information.
-"""
-
-from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
-
-_protocol_handlers = {
-}
-
-def register_transport(prefix, klass, override=True):
-    global _protocol_handlers
-
-    if _protocol_handlers.has_key(prefix):
-        if override:
-            mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
-            _protocol_handlers[prefix] = klass
-    else:
-        mutter('registering transport: %s => %s' % (prefix, klass.__name__))
-        _protocol_handlers[prefix] = klass
-
-class TransportError(BzrError):
-    """All errors thrown by Transport implementations should derive
-    from this class.
-    """
-    def __init__(self, msg=None, orig_error=None):
-        if msg is None and orig_error is not None:
-            msg = str(orig_error)
-        BzrError.__init__(self, msg)
-        self.msg = msg
-        self.orig_error = orig_error
-
-class AsyncError(TransportError):
-    pass
-
-# A set of semi-meaningful errors which can be thrown
-class TransportNotPossible(TransportError):
-    """This is for transports where a specific function is explicitly not
-    possible. Such as pushing files to an HTTP server.
-    """
-    pass
-
-class NonRelativePath(TransportError):
-    """An absolute path was supplied, that could not be decoded into
-    a relative path.
-    """
-    pass
-
-class NoSuchFile(TransportError, IOError):
-    """A get() was issued for a file that doesn't exist."""
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        IOError.__init__(self, errno.ENOENT, self.msg)
-
-class FileExists(TransportError, OSError):
-    """An operation was attempted, which would overwrite an entry,
-    but overwritting is not supported.
-
-    mkdir() can throw this, but put() just overwites existing files.
-    """
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        OSError.__init__(self, errno.EEXIST, self.msg)
-
-class PermissionDenied(TransportError):
-    """An operation cannot succeed because of a lack of permissions."""
-    pass
-
-class ConnectionReset(TransportError):
-    """The connection has been closed."""
-    pass
-
-class AsyncFile(object):
-    """This will be returned from a Transport object,
-    whenever an asyncronous get is requested.
-    """
-    _max_write_buffer = 8192
-
-    def __init__(self):
-        self.read_buffer = []
-        self.write_buffer = []
-        self._finalized = False
-
-    def _total_len(self, buffer):
-        count = 0
-        for b in buffer:
-            count += len(b)
-        return count
-
-    def finalize(self):
-        """This will block until all data has been buffered for
-        reading, or all data has been flushed for writing.
-        Once this is called, you can no longer write more data.
-        """
-        raise NotImplementedError
-
-    def can_read(self):
-        """Can we read some data without blocking"""
-        return len(self.read_buffer) > 0
-
-    def can_write(self):
-        """Can we write some more data out without blocking?"""
-        return (not self._finalized \
-                and self._total_len(self.write_buffer) < self._max_write_buffer)
-
-    def write(self, chunk):
-        if self._finalized:
-            raise AsyncError('write attempted on finalized file.')
-        self.write_buffer.append(chunk)
-
-    def read(self, size=None):
-        if size is None:
-            return ''.join(self.read_buffer)
-        else:
-            out = ''
-            while True:
-                buf = self.read_buffer.pop(0)
-                if len(buf) + len(out) < size:
-                    out += buf
-                else:
-                    left = size - len(out)
-                    out += buf[:left]
-                    buf = buf[left:]
-                    self.read_buffer.insert(0, buf)
-                    return out
-
-
-class Transport(object):
-    """This class encapsulates methods for retrieving or putting a file
-    from/to a storage location.
-
-    Most functions have a _multi variant, which allows you to queue up
-    multiple requests. They generally have a dumb base implementation 
-    which just iterates over the arguments, but smart Transport
-    implementations can do pipelining.
-    In general implementations should support having a generator or a list
-    as an argument (ie always iterate, never index)
-    """
-
-    def __init__(self, base):
-        super(Transport, self).__init__()
-        self.base = base
-
-    def clone(self, offset=None):
-        """Return a new Transport object, cloned from the current location,
-        using a subdirectory. This allows connections to be pooled,
-        rather than a new one needed for each subdir.
-        """
-        raise NotImplementedError
-
-    def should_cache(self):
-        """Return True if the data pulled across should be cached locally.
-        """
-        return False
-
-    def _pump(self, from_file, to_file):
-        """Most children will need to copy from one file-like 
-        object or string to another one.
-        This just gives them something easy to call.
-        """
-        if isinstance(from_file, basestring):
-            to_file.write(from_file)
-        else:
-            from bzrlib.osutils import pumpfile
-            pumpfile(from_file, to_file)
-
-    def _get_total(self, multi):
-        """Try to figure out how many entries are in multi,
-        but if not possible, return None.
-        """
-        try:
-            return len(multi)
-        except TypeError: # We can't tell how many, because relpaths is a generator
-            return None
-
-    def _update_pb(self, pb, msg, count, total):
-        """Update the progress bar based on the current count
-        and total available, total may be None if it was
-        not possible to determine.
-        """
-        if pb is None:
-            return
-        if total is None:
-            pb.update(msg, count, count+1)
-        else:
-            pb.update(msg, count, total)
-
-    def _iterate_over(self, multi, func, pb, msg, expand=True):
-        """Iterate over all entries in multi, passing them to func,
-        and update the progress bar as you go along.
-
-        :param expand:  If True, the entries will be passed to the function
-                        by expanding the tuple. If False, it will be passed
-                        as a single parameter.
-        """
-        total = self._get_total(multi)
-        count = 0
-        for entry in multi:
-            self._update_pb(pb, msg, count, total)
-            if expand:
-                func(*entry)
-            else:
-                func(entry)
-            count += 1
-        return count
-
-    def abspath(self, relpath):
-        """Return the full url to the given relative path.
-        This can be supplied with a string or a list
-        """
-        raise NotImplementedError
-
-    def relpath(self, abspath):
-        """Return the local path portion from a given absolute path.
-        """
-        raise NotImplementedError
-
-    def has(self, relpath):
-        """Does the target location exist?"""
-        raise NotImplementedError
-
-    def has_multi(self, relpaths, pb=None):
-        """Return True/False for each entry in relpaths"""
-        total = self._get_total(relpaths)
-        count = 0
-        for relpath in relpaths:
-            self._update_pb(pb, 'has', count, total)
-            yield self.has(relpath)
-            count += 1
-
-    def get(self, relpath):
-        """Get the file at the given relative path.
-
-        :param relpath: The relative path to the file
-        """
-        raise NotImplementedError
-
-    def get_partial(self, relpath, portion):
-        """Get just part of a file.
-
-        :param relpath: Path to the file, relative to base
-        :param portion: A tuple of [start,length). 
-                        Length can be -1 indicating copy until the end
-                        of the file. If the file ends before length bytes,
-                        just the set of bytes will be returned (think read())
-        :return: A file-like object containing at least the specified bytes.
-                 Some implementations may return objects which can be read
-                 past this length, but this is not guaranteed.
-        """
-        raise NotImplementedError
-
-    def get_partial_multi(self, files, pb=None):
-        """Put a set of files or strings into the location.
-
-        Requesting multiple portions of the same file can be dangerous.
-
-        :param files: A list of tuples of relpath, portion
-                      [(path1, portion1), (path2, portion2),...]
-        :param pb:  An optional ProgressBar for indicating percent done.
-        :return: A generator of file-like objects.
-        """
-        def get_partial(relpath, portion):
-            yield self.get_partial(relpath, portion)
-        self._iterate_over(files, get_partial, pb, 'get_partial', expand=True)
-
-    def get_multi(self, relpaths, pb=None):
-        """Get a list of file-like objects, one for each entry in relpaths.
-
-        :param relpaths: A list of relative paths.
-        :param pb:  An optional ProgressBar for indicating percent done.
-        :return: A list or generator of file-like objects
-        """
-        # TODO: Consider having this actually buffer the requests,
-        # in the default mode, it probably won't give worse performance,
-        # and all children wouldn't have to implement buffering
-        total = self._get_total(relpaths)
-        count = 0
-        for relpath in relpaths:
-            self._update_pb(pb, 'get', count, total)
-            yield self.get(relpath)
-            count += 1
-
-    def put(self, relpath, f):
-        """Copy the file-like or string object into the location.
-
-        :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
-        """
-        raise NotImplementedError
-
-    def put_multi(self, files, pb=None):
-        """Put a set of files or strings into the location.
-
-        :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
-        :param pb:  An optional ProgressBar for indicating percent done.
-        :return: The number of files copied.
-        """
-        def put(relpath, f):
-            self.put(relpath, f)
-        return self._iterate_over(files, put, pb, 'put', expand=True)
-
-    def mkdir(self, relpath):
-        """Create a directory at the given path."""
-        raise NotImplementedError
-
-    def mkdir_multi(self, relpaths, pb=None):
-        """Create a group of directories"""
-        return self._iterate_over(relpaths, self.mkdir, pb, 'mkdir', expand=False)
-
-    def append(self, relpath, f):
-        """Append the text in the file-like or string object to 
-        the supplied location.
-        """
-        raise NotImplementedError
-
-    def append_multi(self, files):
-        """Append the text in each file-like or string object to
-        the supplied location.
-
-        :param files: A set of (path, f) entries
-        :param pb:  An optional ProgressBar for indicating percent done.
-        """
-        return self._iterate_over(files, self.append, pb, 'append', expand=True)
-
-    def copy(self, rel_from, rel_to):
-        """Copy the item at rel_from to the location at rel_to"""
-        raise NotImplementedError
-
-    def copy_multi(self, relpaths, pb=None):
-        """Copy a bunch of entries.
-        
-        :param relpaths: A list of tuples of the form [(from, to), (from, to),...]
-        """
-        # This is the non-pipelined implementation, so that
-        # implementors don't have to implement everything.
-        return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
-
-    def copy_to(self, relpaths, other, pb=None):
-        """Copy a set of entries from self into another Transport.
-
-        :param relpaths: A list/generator of entries to be copied.
-        """
-        # The dummy implementation just does a simple get + put
-        def copy_entry(path):
-            other.put(path, self.get(path))
-
-        return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
-
-
-    def move(self, rel_from, rel_to):
-        """Move the item at rel_from to the location at rel_to"""
-        raise NotImplementedError
-
-    def move_multi(self, relpaths, pb=None):
-        """Move a bunch of entries.
-        
-        :param relpaths: A list of tuples of the form [(from1, to1), (from2, to2),...]
-        """
-        return self._iterate_over(relpaths, self.move, pb, 'move', expand=True)
-
-    def move_multi_to(self, relpaths, rel_to):
-        """Move a bunch of entries to a single location.
-        This differs from move_multi in that you give a list of from, and
-        a single destination, rather than multiple destinations.
-
-        :param relpaths: A list of relative paths [from1, from2, from3, ...]
-        :param rel_to: A directory where each entry should be placed.
-        """
-        # This is not implemented, because you need to do special tricks to
-        # extract the basename, and add it to rel_to
-        raise NotImplementedError
-
-    def delete(self, relpath):
-        """Delete the item at relpath"""
-        raise NotImplementedError
-
-    def delete_multi(self, relpaths, pb=None):
-        """Queue up a bunch of deletes to be done.
-        """
-        return self._iterate_over(relpaths, self.delete, pb, 'delete', expand=False)
-
-    def stat(self, relpath):
-        """Return the stat information for a file.
-        WARNING: This may not be implementable for all protocols, so use
-        sparingly.
-        """
-        raise NotImplementedError
-
-    def stat_multi(self, relpaths, pb=None):
-        """Stat multiple files and return the information.
-        """
-        #TODO:  Is it worth making this a generator instead of a
-        #       returning a list?
-        stats = []
-        def gather(path):
-            stats.append(self.stat(path))
-
-        count = self._iterate_over(relpaths, gather, pb, 'stat', expand=False)
-        return stats
-
-
-    def list_dir(self, relpath):
-        """Return a list of all files at the given location.
-        WARNING: many transports do not support this, so trying avoid using
-        it if at all possible.
-        """
-        raise NotImplementedError
-
-    def lock_read(self, relpath):
-        """Lock the given file for shared (read) access.
-        WARNING: many transports do not support this, so trying avoid using it
-
-        :return: A lock object, which should contain an unlock() function.
-        """
-        raise NotImplementedError
-
-    def lock_write(self, relpath):
-        """Lock the given file for exclusive (write) access.
-        WARNING: many transports do not support this, so trying avoid using it
-
-        :return: A lock object, which should contain an unlock() function.
-        """
-        raise NotImplementedError
-
-
-def transport(base):
-    global _protocol_handlers
-    if base is None:
-        base = '.'
-    for proto, klass in _protocol_handlers.iteritems():
-        if proto is not None and base.startswith(proto):
-            return klass(base)
-    # The default handler is the filesystem handler
-    # which has a lookup of None
-    return _protocol_handlers[None](base)
-
-# Local transport should always be initialized
-import bzrlib.transport.local

=== removed file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/http.py	1970-01-01 00:00:00 +0000
@@ -1,253 +0,0 @@
-#!/usr/bin/env python
-"""\
-An implementation of the Transport object for http access.
-"""
-
-from bzrlib.transport import Transport, register_transport, \
-    TransportNotPossible, NoSuchFile, NonRelativePath, \
-    TransportError
-import os, errno
-from cStringIO import StringIO
-import urllib2
-
-from bzrlib.errors import BzrError, BzrCheckError
-from bzrlib.branch import Branch, BZR_BRANCH_FORMAT
-from bzrlib.trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
-ENABLE_URLGRABBER = False
-
-
-if ENABLE_URLGRABBER:
-    import urlgrabber
-    import urlgrabber.keepalive
-    import urlgrabber.grabber
-    urlgrabber.keepalive.DEBUG = 0
-    def get_url(path, compressed=False):
-        try:
-            url = path
-            if compressed:
-                url += '.gz'
-            mutter("grab url %s" % url)
-            url_f = urlgrabber.urlopen(url, keepalive=1, close_connection=0)
-            if not compressed:
-                return url_f
-            else:
-                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        except urllib2.URLError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-        except urlgrabber.grabber.URLGrabError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-else:
-    def get_url(url, compressed=False):
-        import urllib2
-        if compressed:
-            url += '.gz'
-        mutter("get_url %s" % url)
-        url_f = urllib2.urlopen(url)
-        if compressed:
-            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        else:
-            return url_f
-
-def _find_remote_root(url):
-    """Return the prefix URL that corresponds to the branch root."""
-    orig_url = url
-    while True:
-        try:
-            ff = get_url(url + '/.bzr/branch-format')
-
-            fmt = ff.read()
-            ff.close()
-
-            fmt = fmt.rstrip('\r\n')
-            if fmt != BZR_BRANCH_FORMAT.rstrip('\r\n'):
-                raise BzrError("sorry, branch format %r not supported at url %s"
-                               % (fmt, url))
-            
-            return url
-        except urllib2.URLError:
-            pass
-
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
-        
-class HttpTransportError(TransportError):
-    pass
-
-class HttpTransport(Transport):
-    """This is the transport agent for http:// access.
-    
-    TODO: Implement pipelined versions of all of the *_multi() functions.
-    """
-
-    def __init__(self, base):
-        """Set the base path where files will be stored."""
-        assert base.startswith('http://') or base.startswith('https://')
-        super(HttpTransport, self).__init__(base)
-        # In the future we might actually connect to the remote host
-        # rather than using get_url
-        # self._connection = None
-
-    def should_cache(self):
-        """Return True if the data pulled across should be cached locally.
-        """
-        return True
-
-    def clone(self, offset=None):
-        """Return a new HttpTransport with root at self.base + offset
-        For now HttpTransport does not actually connect, so just return
-        a new HttpTransport object.
-        """
-        if offset is None:
-            return HttpTransport(self.base)
-        else:
-            return HttpTransport(self.abspath(offset))
-
-    def abspath(self, relpath):
-        """Return the full url to the given relative path.
-        This can be supplied with a string or a list
-        """
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
-        baseurl = self.base.rstrip('/')
-        return '/'.join([baseurl] + relpath)
-
-    def relpath(self, abspath):
-        if not abspath.startswith(self.base):
-            raise NonRelativePath('path %r is not under base URL %r'
-                           % (abspath, self.base))
-        pl = len(self.base)
-        return abspath[pl:].lstrip('/')
-
-    def has(self, relpath):
-        """Does the target location exist?
-
-        TODO: HttpTransport.has() should use a HEAD request,
-        not a full GET request.
-        """
-        try:
-            f = get_url(self.abspath(relpath))
-            return True
-        except BzrError:
-            return False
-        except urllib2.URLError:
-            return False
-        except IOError, e:
-            if e.errno == errno.ENOENT:
-                return False
-            raise HttpTransportError(orig_error=e)
-
-    def get(self, relpath, decode=False):
-        """Get the file at the given relative path.
-
-        :param relpath: The relative path to the file
-        """
-        try:
-            return get_url(self.abspath(relpath))
-        except BzrError, e:
-            raise NoSuchFile(orig_error=e)
-        except urllib2.URLError, e:
-            raise NoSuchFile(orig_error=e)
-        except IOError, e:
-            raise NoSuchFile(orig_error=e)
-        except Exception,e:
-            raise HttpTransportError(orig_error=e)
-
-    def put(self, relpath, f):
-        """Copy the file-like or string object into the location.
-
-        :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
-        """
-        raise TransportNotPossible('http PUT not supported')
-
-    def mkdir(self, relpath):
-        """Create a directory at the given path."""
-        raise TransportNotPossible('http does not support mkdir()')
-
-    def append(self, relpath, f):
-        """Append the text in the file-like object into the final
-        location.
-        """
-        raise TransportNotPossible('http does not support append()')
-
-    def copy(self, rel_from, rel_to):
-        """Copy the item at rel_from to the location at rel_to"""
-        raise TransportNotPossible('http does not support copy()')
-
-    def copy_to(self, relpaths, other, pb=None):
-        """Copy a set of entries from self into another Transport.
-
-        :param relpaths: A list/generator of entries to be copied.
-
-        TODO: if other is LocalTransport, is it possible to
-              do better than put(get())?
-        """
-        # At this point HttpTransport might be able to check and see if
-        # the remote location is the same, and rather than download, and
-        # then upload, it could just issue a remote copy_this command.
-        if isinstance(other, HttpTransport):
-            raise TransportNotPossible('http cannot be the target of copy_to()')
-        else:
-            return super(HttpTransport, self).copy_to(relpaths, other, pb=pb)
-
-    def move(self, rel_from, rel_to):
-        """Move the item at rel_from to the location at rel_to"""
-        raise TransportNotPossible('http does not support move()')
-
-    def delete(self, relpath):
-        """Delete the item at relpath"""
-        raise TransportNotPossible('http does not support delete()')
-
-    def async_get(self, relpath):
-        """Make a request for an file at the given location, but
-        don't worry about actually getting it yet.
-
-        :rtype: AsyncFile
-        """
-        raise NotImplementedError
-
-    def list_dir(self, relpath):
-        """Return a list of all files at the given location.
-        WARNING: many transports do not support this, so trying avoid using
-        it if at all possible.
-        """
-        raise TransportNotPossible('http does not support list_dir()')
-
-    def stat(self, relpath):
-        """Return the stat information for a file.
-        """
-        raise TransportNotPossible('http does not support stat()')
-
-    def lock_read(self, relpath):
-        """Lock the given file for shared (read) access.
-        :return: A lock object, which should be passed to Transport.unlock()
-        """
-        # The old RemoteBranch ignore lock for reading, so we will
-        # continue that tradition and return a bogus lock object.
-        class BogusLock(object):
-            def __init__(self, path):
-                self.path = path
-            def unlock(self):
-                pass
-        return BogusLock(relpath)
-
-    def lock_write(self, relpath):
-        """Lock the given file for exclusive (write) access.
-        WARNING: many transports do not support this, so trying avoid using it
-
-        :return: A lock object, which should be passed to Transport.unlock()
-        """
-        raise TransportNotPossible('http does not support lock_write()')
-
-register_transport('http://', HttpTransport)
-register_transport('https://', HttpTransport)
-

=== removed file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-09-15 21:35:53 +0000
+++ bzrlib/transport/local.py	1970-01-01 00:00:00 +0000
@@ -1,198 +0,0 @@
-#!/usr/bin/env python
-"""\
-An implementation of the Transport object for local
-filesystem access.
-"""
-
-from bzrlib.transport import Transport, register_transport, \
-    TransportError, NoSuchFile, FileExists
-import os, errno
-
-class LocalTransportError(TransportError):
-    pass
-
-class LocalTransport(Transport):
-    """This is the transport agent for local filesystem access."""
-
-    def __init__(self, base):
-        """Set the base path where files will be stored."""
-        if base.startswith('file://'):
-            base = base[7:]
-        super(LocalTransport, self).__init__(os.path.realpath(base))
-
-    def should_cache(self):
-        return False
-
-    def clone(self, offset=None):
-        """Return a new LocalTransport with root at self.base + offset
-        Because the local filesystem does not require a connection, 
-        we can just return a new object.
-        """
-        if offset is None:
-            return LocalTransport(self.base)
-        else:
-            return LocalTransport(self.abspath(offset))
-
-    def abspath(self, relpath):
-        """Return the full url to the given relative path.
-        This can be supplied with a string or a list
-        """
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
-        return os.path.join(self.base, *relpath)
-
-    def relpath(self, abspath):
-        """Return the local path portion from a given absolute path.
-        """
-        from branch import _relpath
-        return _relpath(self.base, abspath)
-
-    def has(self, relpath):
-        return os.access(self.abspath(relpath), os.F_OK)
-
-    def get(self, relpath):
-        """Get the file at the given relative path.
-
-        :param relpath: The relative path to the file
-        """
-        try:
-            path = self.abspath(relpath)
-            return open(path, 'rb')
-        except IOError,e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
-            raise LocalTransportError(orig_error=e)
-
-    def put(self, relpath, f):
-        """Copy the file-like or string object into the location.
-
-        :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
-        """
-        from bzrlib.atomicfile import AtomicFile
-
-        try:
-            path = self.abspath(relpath)
-            fp = AtomicFile(path, 'wb')
-        except IOError, e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
-            raise LocalTransportError(orig_error=e)
-        try:
-            self._pump(f, fp)
-            fp.commit()
-        finally:
-            fp.close()
-
-    def mkdir(self, relpath):
-        """Create a directory at the given path."""
-        try:
-            os.mkdir(self.abspath(relpath))
-        except OSError,e:
-            if e.errno == errno.EEXIST:
-                raise FileExists(orig_error=e)
-            elif e.errno == errno.ENOENT:
-                raise NoSuchFile(orig_error=e)
-            raise LocalTransportError(orig_error=e)
-
-    def append(self, relpath, f):
-        """Append the text in the file-like object into the final
-        location.
-        """
-        fp = open(self.abspath(relpath), 'ab')
-        self._pump(f, fp)
-
-    def copy(self, rel_from, rel_to):
-        """Copy the item at rel_from to the location at rel_to"""
-        import shutil
-        path_from = self.abspath(rel_from)
-        path_to = self.abspath(rel_to)
-        try:
-            shutil.copy(path_from, path_to)
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
-
-    def move(self, rel_from, rel_to):
-        """Move the item at rel_from to the location at rel_to"""
-        path_from = self.abspath(rel_from)
-        path_to = self.abspath(rel_to)
-
-        try:
-            os.rename(path_from, path_to)
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
-
-    def delete(self, relpath):
-        """Delete the item at relpath"""
-        try:
-            os.remove(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
-
-    def copy_to(self, relpaths, other, pb=None):
-        """Copy a set of entries from self into another Transport.
-
-        :param relpaths: A list/generator of entries to be copied.
-        """
-        if isinstance(other, LocalTransport):
-            # Both from & to are on the local filesystem
-            # Unfortunately, I can't think of anything faster than just
-            # copying them across, one by one :(
-            import shutil
-
-            total = self._get_total(relpaths)
-            count = 0
-            for path in relpaths:
-                self._update_pb(pb, 'copy-to', count, total)
-                shutil.copy(self.abspath(path), other.abspath(path))
-                count += 1
-            return count
-        else:
-            return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
-
-
-    def async_get(self, relpath):
-        """Make a request for an file at the given location, but
-        don't worry about actually getting it yet.
-
-        :rtype: AsyncFile
-        """
-        raise NotImplementedError
-
-    def list_dir(self, relpath):
-        """Return a list of all files at the given location.
-        WARNING: many transports do not support this, so trying avoid using
-        it if at all possible.
-        """
-        try:
-            return os.listdir(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
-
-    def stat(self, relpath):
-        """Return the stat information for a file.
-        """
-        try:
-            return os.stat(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
-
-    def lock_read(self, relpath):
-        """Lock the given file for shared (read) access.
-        :return: A lock object, which should be passed to Transport.unlock()
-        """
-        from bzrlib.lock import ReadLock
-        return ReadLock(self.abspath(relpath))
-
-    def lock_write(self, relpath):
-        """Lock the given file for exclusive (write) access.
-        WARNING: many transports do not support this, so trying avoid using it
-
-        :return: A lock object, which should be passed to Transport.unlock()
-        """
-        from bzrlib.lock import WriteLock
-        return WriteLock(self.abspath(relpath))
-
-# If nothing else matches, try the LocalTransport
-register_transport(None, LocalTransport)
-register_transport('file://', LocalTransport)

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-09-15 21:35:53 +0000
+++ bzrlib/upgrade.py	2005-09-26 08:56:15 +0000
@@ -15,11 +15,14 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import tempfile, os, errno
-                
+import errno
+import os
+import tempfile
+
 import bzrlib.errors
 import bzrlib.progress
 from bzrlib.xml import serializer_v4
+from bzrlib.osutils import rename
 
 
 def upgrade(branch):
@@ -106,14 +109,6 @@
                 tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',
                     dir=branch.controlfilename('revision-store'))
                 os.close(tmpfd)
-                def special_rename(p1, p2):
-                    if sys.platform == 'win32':
-                        try:
-                            os.remove(p2)
-                        except OSError, e:
-                            if e.errno != errno.ENOENT:
-                                raise
-                    os.rename(p1, p2)
 
                 try:
                     # TODO: We may need to handle the case where the old revision
@@ -121,13 +116,13 @@
 
                     # Remove the old revision entry out of the way
                     rev_path = branch.controlfilename(['revision-store', rev_id+'.gz'])
-                    special_rename(rev_path, tmp_path)
+                    rename(rev_path, tmp_path)
                     branch.revision_store.add(rev_tmp, rev_id) # Add the new one
                     os.remove(tmp_path) # Remove the old name
                     mutter('    Updated revision entry {%s}' % rev_id)
                 except:
                     # On any exception, restore the old entry
-                    special_rename(tmp_path, rev_path)
+                    rename(tmp_path, rev_path)
                     raise
                 rev_tmp.close()
                 updated_revisions.append(rev_id)

=== modified file 'setup.py'
--- setup.py	2005-09-04 03:32:17 +0000
+++ setup.py	2005-09-23 09:25:16 +0000
@@ -6,8 +6,52 @@
 
 from distutils.core import setup
 
+# more sophisticated setup script, based on pychecker setup.py
+import sys, os
+from distutils.command.build_scripts import build_scripts
+
+
+###############################
+# Overridden distutils actions
+###############################
+
+class my_build_scripts(build_scripts):
+    """Customized build_scripts distutils action.
+
+    Create bzr.bat for win32.
+    """
+
+    def run(self):
+        if sys.platform == "win32":
+            bat_path = os.path.join(self.build_dir, "bzr.bat")
+            self.scripts.append(bat_path)
+            self.mkpath(self.build_dir)
+            scripts_dir = self.distribution.get_command_obj("install").\
+                                                            install_scripts
+            self.execute(func=self._create_bat,
+                         args=[bat_path, scripts_dir],
+                         msg="Create %s" % bat_path)
+        build_scripts.run(self) # invoke "standard" action
+
+    def _create_bat(self, bat_path, scripts_dir):
+        """ Creates the batch file for bzr on win32.
+        """
+        try:
+            script_path = os.path.join(scripts_dir, "bzr")
+            bat_str = "@%s %s %%*\n" % (sys.executable, script_path)
+            file(bat_path, "w").write(bat_str)
+            print "file written"
+        except Exception, e:
+            print "ERROR: Unable to create %s: %s" % (bat_path, e)
+            raise e
+
+
+########################
+## Setup
+########################
+
 setup(name='bzr',
-      version='0.0.0',
+      version='0.0.6',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',

=== modified file 'testsweet.py'
--- testsweet.py	2005-09-15 21:35:53 +0000
+++ testsweet.py	2005-09-19 06:05:19 +0000
@@ -55,7 +55,6 @@
     """Indicates that a test was intentionally skipped, rather than failing."""
     # XXX: Not used yet
 
-
 class EarlyStoppingTestResultAdapter(object):
     """An adapter for TestResult to stop at the first first failure or error"""
 
@@ -89,12 +88,9 @@
     def startTest(self, test):
         unittest.TestResult.startTest(self, test)
         # TODO: Maybe show test.shortDescription somewhere?
-        what = test.id()
-        # python2.3 has the bad habit of just "runit" for doctests
-        if what == 'runit':
-            what = test.shortDescription()
+        what = test.shortDescription() or test.id()        
         if self.showAll:
-            self.stream.write('%-60.60s' % what)
+            self.stream.write('%-70.70s' % what)
         self.stream.flush()
 
     def addError(self, test, err):

